from __future__ import annotations

from tcsoa.gen.Cad._2016_09.StructureManagement import FindModelViewsResponse, ReconcilePaletteInput, FindModelViewsInput, NextReconcilePaletteInput, ReconcilePaletteResponse
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def continueFindModelViews(cls, searchID: str, stopFind: bool) -> FindModelViewsResponse:
        """
        Continues a search for model views that was started by the startFindModelViews operation. The input 'searchID'
        specifies which partial find to continue.
        """
        return cls.execute_soa_method(
            method_name='continueFindModelViews',
            library='Cad',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'searchID': searchID, 'stopFind': stopFind},
            response_cls=FindModelViewsResponse,
        )

    @classmethod
    def continueReconcilePalette(cls, reconcilePaletteInput: NextReconcilePaletteInput) -> ReconcilePaletteResponse:
        """
        This operation continues the reconciliation of Fnd0ModelViewProxy object references in a Fnd0ModelViewPalette
        object for a given product structure and configuration. The Fnd0ModelViewProxy objects are authored and managed
        by  CAD applications, and their lifecycle is delegated to their CAD owning model's lifecycle.  As the CAD
        owning models are updated, so are their Fnd0ModelViewProxy objects.  However, the Fnd0ModelViewPalette objects
        that reference these Fnd0ModelViewProxy objects are not automatically updated during CAD model update in all
        cases, because there are times when this update needs to be carefully controlled by users.  Consequently, the
        Fnd0ModelViewPalette can get out of synchronization with the structure it is attached to.  This service
        operation is provided to help reconcile differences between the loaded structure configuration and an existing
        Fnd0ModelViewPalette in a user controlled manner.  It provides the caller with information about which
        Fnd0ModelViewProxy objects should be removed and which should be replaced by analyzing the current configured
        structure and comparing it to what was found in an existing Fnd0ModelViewPalette.
        The Fnd0ModelViewPalette contains Fnd0ModelViewProxy object references for  components belonging to the product
        structure. When the components in the structure gets revised, cloned, removed, or a model view is deleted or
        unpublished, or when the structure configuration gets changed, the Fnd0ModelViewPalette needs to be reconciled
        against the current structure in order to get the proper Fnd0ModelViewProxy objects corresponding to the
        updated structure. Using this operation, a list of Fnd0ModelViewProxy objects that need to be replaced or
        removed for a given Fnd0ModelViewPalette can be identified, retrieved, and presented to user, so that the user
        can carefully control the update of the Fnd0ModelViewPalette for the various update use cases.
        This operation is designed in such a way that the caller can use it in a threaded operation and display results
        in batches instead of waiting for the entire reconciliation process to be complete which may take a fair amount
        of time for large structures. Before invoking this this operation, the 'startReconcilePalette' operation should
        have been invoked for the same 'clientID'.
        Note: The reconcile process has to be closed by the caller using the 'continueReconcilePalette' operation with
        'stopReconcile' as true. For example the caller calls 'startReconcilePalette' operation followed by
        'continueReconcilePalette' operation. Once the finished variable is returned as true in the response the caller
        calls 'continueReconcilePalette' with 'stopReconcile' as true so that the resources are freed up in the server.
        
        Use cases:
        The operation supports the following use cases.
        Use Case 1 :  Update a Fnd0ModelViewPalette per informal engineering change (i.e. overwrite)
        1.An authoring user creates a detailed design in a CAD application.  The user creates and publishes Model Views
        describing the design details. The Model Views are persisted in Teamcenter as Fnd0ModelViewProxy objects using
        the operation 'createOrUpdateModelViewProxies'.
        2.The authoring user opens a configured product structure in a visualization enabled Teamcenter application and
        finds the published Fnd0ModelViewProxy objects in the product structure using the operation
        'findModelViewsInStructure'.
        3.The authoring user selects a limited subset of the Fnd0ModelViewProxy objects found, adds them to a Palette,
        organizes them for presentation by re-ordering and grouping, and creates a Fnd0ModelViewPalette object in
        Teamcenter which references Fnd0ModelViewProxy objects using the operation 'createOrUpdateModelViewPalette'.
        4.A reviewing user opens the product structure configuration along with the Fnd0ModelViewPalette into a
        visualization enabled Teamcenter application, reviews each Model View for accuracy, creates review comments for
        changes that need to be made, and submits those changes to the CAD Designer that authored the original detailed
        design.
        5.The authoring user receives the review comments from various reviewing users, and uses the CAD application to
        make changes to the CAD model(s) such as add or remove components, reposition or change parts, add, delete, or
        change model views, publish or unpublish model views, etc.  The updated CAD models are saved to Teamcenter
        along with updated Fnd0ModelViewProxy objects using the operation 'createOrUpdateModelViewProxies'.  Since this
        is informal change, the CAD models are typically overwritten as opposed to revised.
        6.The authoring user opens the configured product structure with attached Fnd0ModelViewPalette created in step
        3 in a visualization enabled Teamcenter application, and some of the Fnd0ModelViewProxy objects referenced by
        the Fnd0ModelViewPalette are no longer valid relative to the current structure due to the CAD model changes. 
        The user invokes the palette reconcile action which triggers the operation 'startReconcilePalette' followed by
        'continueReconcilePalette'. The results are displayed to the user providing information on which
        Fnd0ModelViewProxy objects are still valid, which need to be replaced, and which need to be removed in order to
        update the Palette.  The results are streamed to the client  in batches, so the user is updated on progress and
        can choose to stop the reconcile operation at any time.
        7.The authoring user updates the Fnd0ModelViewPalette per the changes suggested by this operation, and saves
        the palette updates using the operation 'createOrUpdateModelViewPalette'.
        
        Use Case 2: Update a Fnd0ModelViewPalette to a different structure configuration
        1.The user opens a structure configuration different from that used to author the original Fnd0ModelViewPalette
        (per use case 1), and sends this to the visualization enabled palette authoring tool in Teamcenter.
        2.The user opens the Fnd0ModelViewPalette and invokes the palette reconcile action which triggers the operation
        'startReconcilePalette' followed by 'continueReconcilePalette'. The results are displayed to the user providing
        which Fnd0ModelViewProxy objects cannot be found, which are still valid, which need to be replaced, and which
        need to be removed in order to reconcile the Palette to the current structure configuration. The results are
        streamed to the client  in batches, so the user is updated on progress and can choose to stop the reconcile
        operation at any time.
        3.The authoring user updates the Fnd0ModelViewPalette per the changes suggested by the operation, and saves the
        palette updates using the operation 'createOrUpdateModelViewPalette'. The updated structure configuration
        information used for the updated palette is stored with the palette. 
        
        Use Case 3 : Update a Fnd0ModelViewPalette per formal engineering change (i.e. Revise)
        1.The user of a CAD application makes changes to the CAD model(s) representing the detailed design such as
        revising components, add, delete, or change model views, publish or unpublish model views, etc during revise of
        a detailed design.  The revised CAD models are saved to Teamcenter, where new and updated Fnd0ModelViewProxy
        objects are published using the operation 'createOrUpdateModelViewProxies' and Teamcenter deep copy rules come
        into play.
        2.The user opens the configured product structure with attached Fnd0ModelViewPalette (see use case 1) in a
        visualization enabled Teamcenter application, and some of the Fnd0ModelViewProxy objects referenced by the
        Fnd0ModelViewPalette are no longer valid due to the revised CAD model(s).  Some of the Fnd0ModelViewProxy
        objects in the Fnd0ModelViewPalette are from previous revisions of the revised components, others cannot be
        found. The user invokes the palette reconcile action which triggers the operation 'startReconcilePalette'
        followed by 'continueReconcilePalette'. The results are displayed to the user providing information on which
        Fnd0ModelViewProxy objects are still valid, which need to be replaced with Fnd0ModelViewProxy objects on new
        revisions of the CAD models, and which need to be removed in order to update the Palette.  The results are
        streamed to the client  in batches, so the user is updated on progress and can choose to stop and restart the
        reconcile operation at any time.
        3.The user updates the Fnd0ModelViewPalette per the changes suggested by this operation, and saves the updated
        palette using the operation 'createOrUpdateModelViewPalette'.
        """
        return cls.execute_soa_method(
            method_name='continueReconcilePalette',
            library='Cad',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'reconcilePaletteInput': reconcilePaletteInput},
            response_cls=ReconcilePaletteResponse,
        )

    @classmethod
    def startFindModelViews(cls, findInput: FindModelViewsInput) -> FindModelViewsResponse:
        """
        Finds Model View Proxy (Fnd0ModelViewProxy) objects associated to any objects within the specified structure.
        This operation is most often used when creating a disclosure object.
        
        Objects that act as a disclosure are Items or Worksets that have a relation of Fnd0DisclosingObject to the
        actual design objects being disclosed and hence intend to have a list of disclosed Model View Proxy objects.
        
        Used in conjunction with the continueFindModelViews operation. If the preference MVFindMinNodeCount has a
        value, that value is the minimum number of structure nodes to search before returning with 'finished' as false.
        
        Use cases:
        This API supports the following use cases:
        Use Case 1: Creation of a new design disclosure
        
        The operation can be used for supporting creation of a disclosure object. The disclosure object will most
        commonly be a Workset (Cpd0Workset subtype) but may also be a specific type of Item Revision. The actual
        disclosure object may be pre-existing but would not be acting as a disclosure until this operation creates and
        attaches the desired list of disclosed model view references.
        
        The purpose of a design disclosure is to act as a 3D equivalent of a 2D drawing. This disclosure object will
        collect all geometry including background geometry if necessary and PMI needed to show all views describing the
        detailed design for the object being disclosed. The disclosure content may be organized into the following item
        revisions or subsets (depending on the disclosure type):
        Foreground content - actual installation assembly reference
        Background content - context of a product into which the installation assembly is used.
        Separately collected geometry - such as welds between the foreground and background objects. 
        
        To create the correct model view list (see the createOrUpdateModelViewPalette service operation), the client
        and user must first find candidate model view proxies from which to choose the necessary proxies to disclose.
        
        During the operation, a designer would create a Workset to collect all geometry needed to support  installation
        assembly PMI. It is done to collect assemblies being installed and where they are being installed so that PMI
        and model views associated with this combination can be authored and then disclosed. The following types of
        geometry may be collected in a single Workset:
        Foreground Subset
        Background Subset
        ItemRevision (Weld Collector)
        
        Multiple CAD Designers will be concurrently authoring PMI and Model Views (MVs) for disclosure at multiple
        levels of the sub-assemblies under the installation assembly. All the MVs authored at this time are
        undisclosed. However, some of them will be marked as a candidates for disclosure.
        Owning model (ItemRevision) must exist prior to creating the MV proxy in teamcenter by a new service operation
        createOrUpdateModelViewProxies. The owing model will be specified as a request parameter for each MV proxy.
        
        A visualization user begins Disclosure authoring by retrieving the above mentioned collector workset and making
        sure it has disclosing object (Fnd0DisclosingObject) relations to the actual installation item revisions whose
        design is being disclosed. They will then use the startFindModelViews operation followed by the
        continueFindModelViews operation to find model views that are marked as candidates for disclosure and then use
        a third operation (createOrUpdateModelViewPalette) to create the list of actually disclosed model views to be
        persisted for the Disclosure being updated. 
        Note:
        Candidate Disclosed Model Views will be retrievedfrom all structure nodes that are not suppressed or children
        of suppressed nodes, and then filtered only by any provided values in the 'withDisclosureIntents'.
        """
        return cls.execute_soa_method(
            method_name='startFindModelViews',
            library='Cad',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'findInput': findInput},
            response_cls=FindModelViewsResponse,
        )

    @classmethod
    def startReconcilePalette(cls, reconcilePaletteInput: ReconcilePaletteInput) -> ReconcilePaletteResponse:
        """
        This operation starts the reconciliation of Fnd0ModelViewProxy object references in a Fnd0ModelViewPalette
        object for a given product structure and configuration. The Fnd0ModelViewProxy objects are authored and managed
        by  CAD applications, and their lifecycle is delegated to their CAD owning model's lifecycle.  As the CAD
        owning models are updated, so are their Fnd0ModelViewProxy objects.  However, the Fnd0ModelViewPalette objects
        that reference these Fnd0ModelViewProxy objects are not automatically updated during CAD model update in all
        cases, because there are times when this update needs to be carefully controlled by users.  Consequently, the
        Fnd0ModelViewPalette can get out of synchronization with the structure it is attached to. This service
        operation is provided to help reconcile differences between the loaded structure configuration and an existing
        Fnd0ModelViewPalette in a user controlled manner.  It provides the caller with information about which
        Fnd0ModelViewProxy objects should be removed and which should be replaced by analyzing the current configured
        structure and comparing it to what was found in an existing Fnd0ModelViewPalette.
        The Fnd0ModelViewPalette contains Fnd0ModelViewProxy object references for  components belonging to the product
        structure. When the components in the structure gets revised, cloned, removed, or a model view is deleted or
        unpublished, or when the structure configuration gets changed, the Fnd0ModelViewPalette needs to be reconciled
        against the current structure in order to get the proper Fnd0ModelViewProxy objects corresponding to the
        updated structure.  Using this operation, a list of Fnd0ModelViewProxy objects that need to be replace or
        removed for a given Fnd0ModelViewPalette can be identified, retrieved and presented to user, so that the user
        can carefully control the update of the Fnd0ModelViewPalette for the various update use cases.
        This operation is designed in such a way that the caller can use it in a threaded operation and display results
        in batches instead of waiting for the entire reconciliation process to be complete which may take a fair amount
        of time for large structures. This operation is thus supplemented by 'continueReconcilePalette' operation.
        Note: The reconcile process has to be closed by the caller using the 'continueReconcilePalette' operation with
        'stopReconcile' as true. For example the caller calls 'startReconcilePalette' operation followed by
        'continueReconcilePalette' operation. Once the finished variable is returned as true in the response the caller
        calls 'continueReconcilePalette' with 'stopReconcile' as true so that the resources are freed up in the server.
        
        Use cases:
        The operation supports the following use cases.
        Use Case 1 : Update a Fnd0ModelViewPalette per informal engineering change (i.e. overwrite)
        1. An authoring user creates a detailed design in a CAD application.  The user creates and publishes Model
        Views describing the design details. The Model Views are persisted in Teamcenter as Fnd0ModelViewProxy objects
        using the operation 'createOrUpdateModelViewProxies'.
        2. The authoring user opens a configured product structure in a visualization enabled Teamcenter application
        and finds the published Fnd0ModelViewProxy objects in the product structure using the operation
        'findModelViewsInStructure'.
        3. The authoring user selects a limited subset of the Fnd0ModelViewProxy objects found, adds them to a Palette,
        organizes them for presentation by re-ordering and grouping, and creates a Fnd0ModelViewPalette object in
        Teamcenter which references Fnd0ModelViewProxy objects using the operation 'createOrUpdateModelViewPalette'.
        4. A reviewing user opens the product structure configuration along with the Fnd0ModelViewPalette into a
        visualization enabled Teamcenter application, reviews each Model View for accuracy, creates review comments for
        changes that need to be made, and submits those changes to the CAD Designer that authored the original detailed
        design.
        5. The authoring user receives the review comments from various reviewing users, and uses the CAD application
        to make changes to the CAD model(s) such as add or remove components, reposition or change parts, add, delete,
        or change model views, publish or unpublish model views, etc.  The updated CAD models are saved to Teamcenter
        along with updated Fnd0ModelViewProxy objects using the operation 'createOrUpdateModelViewProxies'.  Since this
        is informal change, the CAD models are typically overwritten as opposed to revised.
        6. The authoring user opens the configured product structure with attached Fnd0ModelViewPalette created in step
        3 in a visualization enabled Teamcenter application, and some of the Fnd0ModelViewProxy objects referenced by
        the Fnd0ModelViewPalette are no longer valid relative to the current structure due to the CAD model changes. 
        The user invokes the palette reconcile action which triggers the operation 'startReconcilePalette' followed by
        'continueReconcilePalette'. The results are displayed to the user providing information on which
        Fnd0ModelViewProxy objects are still valid, which need to be replaced, and which need to be removed in order to
        update the Palette.  The results are streamed to the client  in batches, so the user is updated on progress and
        can choose to stop the reconcile operation at any time.
        7. The authoring user updates the Fnd0ModelViewPalette per the changes suggested by this operation, and saves
        the palette updates using the operation 'createOrUpdateModelViewPalette'.
        
        Use Case 2: Update a Fnd0ModelViewPalette to a different structure configuration
        1.The user opens a structure configuration different from that used to author the original Fnd0ModelViewPalette
        (per use case 1), and sends this to the visualization enabled palette authoring tool in Teamcenter.
        2.The user opens the Fnd0ModelViewPalette and invokes the palette reconcile action which triggers the operation
        'startReconcilePalette' followed by 'continueReconcilePalette'. The results are displayed to the user providing
        which Fnd0ModelViewProxy objects cannot be found, which are still valid, which need to be replaced, and which
        need to be removed in order to reconcile the Palette to the current structure configuration.  The results are
        streamed to the client  in batches, so the user is updated on progress and can choose to stop the reconcile
        operation at any time.
        3.The authoring user updates the Fnd0ModelViewPalette per the changes suggested by the operation, and saves the
        palette updates using the operation 'createOrUpdateModelViewPalette'. The updated structure configuration
        information used for the palette update is stored with the palette.
        
        Use Case 3 : Update a Fnd0ModelViewPalette per formal engineering change (i.e. Revise)
        1.The user of a CAD application makes changes to the CAD model(s) representing the detailed design such as
        revising components, add, delete, or change model views, publish or unpublish model views, etc during revise of
        a detailed design.  The revised CAD models are saved to Teamcenter, where new and updated Fnd0ModelViewProxy
        objects are published using the operation 'createOrUpdateModelViewProxies' and Teamcenter deep copy rules come
        into play.
        2.The user opens the configured product structure with attached Fnd0ModelViewPalette (see use case 1) in a
        visualization enabled Teamcenter application, and some of the Fnd0ModelViewProxy objects referenced by the
        Fnd0ModelViewPalette are no longer valid due to the revised CAD model(s).  Some of the Fnd0ModelViewProxy
        objects in the Fnd0ModelViewPalette are from previous revisions of the revised components, others cannot be
        found. The user invokes the palette reconcile action which triggers the operation 'startReconcilePalette'
        followed by 'continueReconcilePalette'. The results are displayed to the user providing information on which
        Fnd0ModelViewProxy objects are still valid, which need to be replaced with Fnd0ModelViewProxy objects on new
        revisions of the CAD models, and which need to be removed in order to update the Palette.  The results are
        streamed to the client  in batches, so the user is updated on progress and can choose to stop and restart the
        reconcile operation at any time.
        3.The user updates the Fnd0ModelViewPalette per the changes suggested by this operation, and saves the updated
        palette using the operation 'createOrUpdateModelViewPalette'.
        """
        return cls.execute_soa_method(
            method_name='startReconcilePalette',
            library='Cad',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'reconcilePaletteInput': reconcilePaletteInput},
            response_cls=ReconcilePaletteResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class CreateProxyLinkInputInfo(TcBaseObj):
    """
    A structure containing uid,owning site of the object for which the ProxyLink object is created, primary object to
    which the proxy link object is related, relation name used to relate the primary object and the proxylink and the
    attributes to be populated on the proxy link object.
    
    :var objectUID: The uid of the object for which the ProxyLink is created.
    :var owningSite: The owning site of the object for which the ProxyLink is created.
    :var proxyLinkType: The name of the ProxyLink type.
    :var attributes: A map (string/string) of  property names and initial  value pairs to be populated on the ProxyLink
    object. 
    :var primaryObject: The BusinessObject to attach the ProxyLink object. 
    :var relationName: The name of the relation to use while attaching the ProxyLink object to the primaryObject .
    """
    objectUID: str = ''
    owningSite: str = ''
    proxyLinkType: str = ''
    attributes: NameValuePairs = None
    primaryObject: BusinessObject = None
    relationName: str = ''


"""
A map (string/string) of  property names and initial  value pairs to be populated on the ProxyLink object.
"""
NameValuePairs = Dict[str, str]



from tcsoa.gen.Notification._2014_10.services import SubscriptionManagementService as imp0
from tcsoa.utils import TcService


class SubscriptionManagementService(TcService):
    createSubscriptions = imp0.createSubscriptions
    findSubscriptions = imp0.findSubscriptions
    getSubscriptions = imp0.getSubscriptions
    modifySubscriptions = imp0.modifySubscriptions



from __future__ import annotations

from tcsoa.gen.Internal.Core._2007_06.ProjectLevelSecurity import GetFilteredProjectDataResponse, ProjectSmartFolderHierarchyOutputResponse, GetFilteredProjectDataInputData, TopLevelHierarchyOutputResponse
from typing import List
from tcsoa.utils import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getProjectsSmartFolderHierarchy(cls, projectIDs: List[str]) -> ProjectSmartFolderHierarchyOutputResponse:
        """
        This operation gets a list of  project smart folder hierarchy for each of the given project ids. If the project
        ID given in the argument does not exists error code 101007: the project ID is invalid; will be returned in a
        partial error.
        """
        return cls.execute_soa_method(
            method_name='getProjectsSmartFolderHierarchy',
            library='Internal-Core',
            service_date='2007_06',
            service_name='ProjectLevelSecurity',
            params={'projectIDs': projectIDs},
            response_cls=ProjectSmartFolderHierarchyOutputResponse,
        )

    @classmethod
    def getTopLevelSmartFolderHierarchy(cls) -> TopLevelHierarchyOutputResponse:
        """
        This operation returns top-level smart folder hierarchy as configured by the administrator.  For more
        information on smart folder hierarchy refer to Teamcenter documentation under administration section of Project
        Level Security. If the global constant named "ProjectTopLevelSmartFolders" does not exist in the system then
        the operation returns an empty list. There are no errors that will be returned.
        """
        return cls.execute_soa_method(
            method_name='getTopLevelSmartFolderHierarchy',
            library='Internal-Core',
            service_date='2007_06',
            service_name='ProjectLevelSecurity',
            params={},
            response_cls=TopLevelHierarchyOutputResponse,
        )

    @classmethod
    def getFilteredProjectData(cls, input: List[GetFilteredProjectDataInputData]) -> GetFilteredProjectDataResponse:
        """
        This operation obtains data contained in a specified project by applying the given filter criteria. If no
        filter criteria is specified in the input argument all data in the project is returned. If the project id
        specified in the argument does not exist then s error code 101007: the project ID is invalid is returned and if
        the specified type does not exist in the system and error 39007: The specified name type name is invalid for a
        type is returned.
        """
        return cls.execute_soa_method(
            method_name='getFilteredProjectData',
            library='Internal-Core',
            service_date='2007_06',
            service_name='ProjectLevelSecurity',
            params={'input': input},
            response_cls=GetFilteredProjectDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.Requirementsmanagement._2007_01.RequirementsManagement import SetRichContentResponse, CreateOrUpdateResponse, GetRichContentResponse, SetContentInput, ExportToApplicationResponse, ImportFromApplicationResponse, ImportFromApplicationInputData, PartInfo, GetContentInput, ExportToApplicationInputData
from typing import List
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def getRichContent(cls, inputs: List[GetContentInput]) -> GetRichContentResponse:
        """
        getRichContent operation is used to retrieve rich text contents of SpecElement type of objects which is a
        subclass of WorkspaceObject.  A .docm file is generated as a result of getRichContent operation which can be
        opened in MSWord.
        
        Use cases:
        User can open content (body text) in word for view and edit purpose.
        
        Exceptions:
        >If there is any error during generating transient file read ticket due to a configuration issue at the server,
        then the operation throws a service exception. Example- If the transient volume directory is not configured at
        the server then the FMS fails to generate a file at the server and subsequent file download operation fails. In
        such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='getRichContent',
            library='Requirementsmanagement',
            service_date='2007_01',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=GetRichContentResponse,
        )

    @classmethod
    def importFromApplication(cls, inputs: List[ImportFromApplicationInputData]) -> ImportFromApplicationResponse:
        """
        This operation is used for importing the contents of the given MSWord document  to create objects of type
        SpecElement.  The MSWord document to be imported to Teamcenter should have .docx file format. If the
        application format is MSWordXML then the operation parses the given MSWord document to creates a structure of
        chosen subtype of SpecElement.
        
        Objects of type Requirement/Paragraph are created at the server after importing the document.  When this
        operation is called from Teamcenter rich client, a structure is created and is opened in the
        'RequirementsManagement' application.  The structure and indentation of the child Requirement is driven by the
        MSWord outline level in the document.  Each Paragraph that is formatted in an outline level style produces a
        separate Requirement. This file may be located in a local file system folder or a network folder.  This
        operation supports MSWord documents in MS Office Open XML format (.docx format) only.
        
        Use cases:
        User can create an MSWord document and import it using this operation to create a structure. Each Paragraph in
        MSWord document represents a Requirement/Paragraph in the structure.
        
        Exceptions:
        >If there is any error during generating transient file ticket due to a configuration issue at the server, then
        the operation throws a service exception. For example If the transient volume directory is not configured at
        the server then the FMS fails to import the document and subsequent file upload operation fails. In such
        situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='importFromApplication',
            library='Requirementsmanagement',
            service_date='2007_01',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=ImportFromApplicationResponse,
        )

    @classmethod
    def setRichContent(cls, inputs: List[SetContentInput]) -> SetRichContentResponse:
        """
        The SOA operation is responsible for setting rich text contents from Word document to a Fulltext object of
        requirement. This SOA operation will be called when; User modified rich content of requirements through word
        document. This operation will basically accept Fulltext object to process in "objectToProcess" variable. Along
        with Fulltext object, this operation will accept transient file tickets for MSWord Document which is modified
        by user. All exceptions are added to service data, if occurred.
        
        Use cases:
        User can set rich text contents of SpecElement object by using setRichContent SOA.
        
        Exceptions:
        >If there is any error while downloading file then the operation throws a service exception. Example- The
        transient volume directory is not configured at the server then the FMS fails to generate a file at the server
        and subsequent file download operation fails. In such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='setRichContent',
            library='Requirementsmanagement',
            service_date='2007_01',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=SetRichContentResponse,
        )

    @classmethod
    def createOrUpdate(cls, info: List[PartInfo]) -> CreateOrUpdateResponse:
        """
        This operation creates objects of type Item .  The related objects such as ItemRevision, Dataset and Forms are
        also created during this operation.  This operation checks for the existence of the Item, ItemRevision, and
        Dataset.  If the Item and ItemRevision already exists, but the Dataset does not exist, then the Dataset is
        created.  If the Dataset exists, a new version will be added to the existing version.  If any of the objects
        exists, but the input attribute values that differ from those already set, attempts are made to update the
        values. There is no attempt to query and update an existing object without a reference to that object. This
        operation has the additional behavior to create and update the Dataset along with the creation of Item.
        
        Use cases:
        User can create objects of type Item using this operation.
        User can create or update objects of type Dataset using this operation.
        
        Exceptions:
        >None.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdate',
            library='Requirementsmanagement',
            service_date='2007_01',
            service_name='RequirementsManagement',
            params={'info': info},
            response_cls=CreateOrUpdateResponse,
        )

    @classmethod
    def exportToApplication(cls, inputs: List[ExportToApplicationInputData]) -> ExportToApplicationResponse:
        """
        This operation is used for exporting Teamcenter objects (WorkspaceObject) to applications like MSWord and
        MSExcel.  Teamcenter business objects should already be created so that the objects can be exported to MSWord
        and MSExcel using the service operation.
        
        An Excel (.mhtml) or a Word (.docx) file is generated at the server as a result of this operation.  The
        generated file will contain Teamcenter objects and their properties. Depending upon the application format that
        is being passed as input parameter, the generated file can be opened in MSWord or MSExcel.  If the application
        format is MSWordXML then the generated document is a Word document.  If the application format is MSExcel then
        the generated sheet is a static Excel spreadsheet.  If the application format is MSExcelLive then the generated
        sheet is a Live Excel spreadsheet. "Live" sheet means that modifications can be done to the data in MSExcel
        which will reflect to Teamcenter. 
        
        
        Use cases:
        User can create Teamcenter objects in RAC and then export those objects and their properties to MSWord or
        MSExcel. If "Live" option is selected then User can create "Live" spreadsheet after export to MSExcel.
        
        Following usecases are supported in this operation
        
        - Export of Workspace objects to MSWord (static)
        - Export of Teamcenter objects to MSExcel(static)
        - Export of Teamcenter objects to MSExcel(Live) and edit the properties from Excel Live sheet.
        
        
        
        Exceptions:
        >If there is any error during generating transient file read ticket due to a configuration issue at the server,
        then the operation throws a service exception. Example- If the transient volume directory is not configured at
        the server then the FMS fails to generate a file at the server and subsequent file download operation fails. In
        such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='Requirementsmanagement',
            service_date='2007_01',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=ExportToApplicationResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportManufaturingFeaturesInput(TcBaseObj):
    """
    input for import manufaturing features.
    
    :var context: A BOMLine in the product structure. The connected parts of the imported manufatruing features are
    searched under this line.
    :var content: A list of detailed input.
    """
    context: BusinessObject = None
    content: List[ImportManufaturingFeaturesScope] = ()


@dataclass
class ImportManufaturingFeaturesResponse(TcBaseObj):
    """
    The output of import manufacturing features.
    
    :var logFileName: The name of the generated log file.
    :var logFileTicket: The FMS ticket of the log file.
    :var serviceData: The service data.
    """
    logFileName: str = ''
    logFileTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class ImportManufaturingFeaturesScope(TcBaseObj):
    """
    Detailed input for importing manufacturing features
    
    :var xmlFileTicket: The FMS file ticket for the input XML file to be imported.
    :var container: The BOMLine under which the manufaturing features are going to be imported.
    :var importMode: Indicates whether the existing manufaturing features under the container maybe deleted or not. The
    possible values of the import mode are: keepExistingFeatures - The existing discrete manufacturing features under
    the container should not be deleted. refreshWholeContainer - The existing discrete manufacturing features under the
    container may be deleted.
    :var nameRefFileTickets: The FMS tickets for dataset named reference files.
    """
    xmlFileTicket: str = ''
    container: BusinessObject = None
    importMode: str = ''
    nameRefFileTickets: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateNextValuesIn(TcBaseObj):
    """
    The input required to generate the next values.
    
    :var clientId: A unique string supplied by the caller.
    :var operationType: The type of the operation for which the values are to be generated.
    1-CREATE
    2-REVISE
    3-SAVEAS.
    
    An error 74007 is returned by the generateNextValues operation if the operationType value is not in any of the
    above. 
    :var businessObjectName: The name of the business object for which the property values are to be generated. An
    error 39007 is returned by the generateNextValues operation if the type name is not valid.
    :var propertyNameWithSelectedPattern:  A map of  property name and  naming rule pattern  pairs( string/string ).The
    key is the property name for which the value is to be generated and the value is the naming rule pattern string
    selected for  that property. If no pattern is selected for the property  the pattern string  should be empty
    string. The caller should only pass the properties that are autoassignable.
    :var propValues: A map of property name and values list(string/vector) of all the properties that are provided as
    inputs to create/revise/save as action of an object excluding the properties whose values are needed to be
    generated.If a multi field key is configured on the object ,the values of the constituent properties of the multi
    field key other than the ones for which the values are currently being generated are obtained from this map in
    order to perform the validation.
    The entries in this map are also used to generate values for the autoassignable properties of some business objects.
    
    Eg:
    Identifier: "idContext" value is needed to generate "idfr_id" property value
    
    An error  74032 is returned if any of the above property values are not provided .
    
    The values of the properties are to be provided as strings. For non string type properties use the
    Property.toXXXString functions (ie. toDateString) to covert the actual values to a string.
    
    :var additionalInputParams: A map of extra information which is required by user exits to generate values.
    A map where the key (string) is a parameter name and the value (string) is the parameter value.
    
    Valid parameters are:
    
    "ruleSuffix":
    This is only used when Baseline action to be performed. 'PDR'  should be passed  as ruleSuffix  in case of Baseline
    action. An error 74030 is returned if invalid ruleSuffix is supplied.
    
    "sourceObject":
    The object that is being revised/saved  or  the parent object  during  Dataset creation . Its value should be
    passed as the object UID. An error 74031 is returned if this parameter is not provided during revise/save as
    action.  It is optional in case of  Dataset creation .
    
    Object Name        |        Operation                |            Value                    |
    ============================================|
        ITEM                |        Create                    |        NULLTAG                |
                            |        SaveAs                    |        Source tag                |
                            |                                    |                                    |
        ITEM Revision    |        Create/SaveAs            |        NULLTAG                |
                            |        Revise                    |        Source tag                |
                            |        Baseline                    |        Source tag                |
                             |                                    |                                     |
        Identifier            |            Create                |            NULLTAG             |    
                            |        Revise/SaveAs            |           Source tag             |
                            |                                    |                                     |
        Dataset            |           Create                    |           Source tag             |
                            |           SaveAs                |            NULLTAG             |
                            |                                    |                                     |
        CPD Objects    |    Create/Revise/SaveAs    |           Source tag             |
                            |                                    |                                     |
    ============================================|
    
    Any parameter other than the above will not be considered. 
    
    :var compoundObjectInput: Map for compound property.
    A map where the key (string) is a compound property name and the value (vector) is vector of GenerateNextValuesIn
    type.
    
    By using this map user can generate values for its own properties and also values for any of its   compound
    objects's propert. Provided property is autoassignable.
    
    For example user can generate value for "item_id" on item Object and value for "item_revision_id" property also
    using its compound pbject ItemRevision using this map.
    """
    clientId: str = ''
    operationType: int = 0
    businessObjectName: str = ''
    propertyNameWithSelectedPattern: PropertyNameWithSelectedPatternMap = None
    propValues: PropValuesMap = None
    additionalInputParams: AdditionalInputParamsMap = None
    compoundObjectInput: CompoundObjectInputMap = None


@dataclass
class GenerateNextValuesResponse(TcBaseObj):
    """
    This output structure contains service data with partial errors and a list of GeneratedValuesOutput each for an
    entry in the generateNextValuesIn input list.
    
    :var data: The Service Data with partial errors for each GenerateNextValuesInput and identified by  its clientId.
    :var generatedValues: A list of GeneratedValuesOutput one for each entry in generateNextvaluesIn input list and 
    identified by its clientId.
    """
    data: ServiceData = None
    generatedValues: List[GeneratedValuesOutput] = ()


@dataclass
class GeneratedValue(TcBaseObj):
    """
    This structure contains the  generated value for the input property. It also contains the information whether or
    not the generated values can be modified in the user interface.
    
    :var errorCode: The error encountered during the generation of value.
    :var nextValue: The  value generated based on the user exit/naming rule configured on the property.If the property
    has multiple naming rule patterns with counters  and if the selected pattern is passed as empty string, the value
    is generated based on the first pattern configured on the naming rule. An error 74006 is returned if the generated
    value does not match the naming rule pattern configured on the property and the nextValue is set to empty string.
    An error 74006 is returned if the generated value does not match the naming rule pattern configured on the property
    and the nextValue is set to empty string.
    :var isModifiable: True if the user can modify the generated value.If isModifiable is false the attempts to change
    the generated value shall be prevented at the user interface.
    """
    errorCode: int = 0
    nextValue: str = ''
    isModifiable: bool = False


@dataclass
class GeneratedValuesOutput(TcBaseObj):
    """
    This structure contains the values generated for each of the properties supplied in  the
    propertyNameWithSelectedPattern map of corresponding PropertyValuesInput structure.
    
    :var clientId: Identifying string from the source GenerateNextValuesInput.
    :var generatedValues: This contains map of property name and its values.
    :var generatedValuesOfSecondaryObjects: Field contains value for generated secondary object property values.
    """
    clientId: str = ''
    generatedValues: GeneratedValuesMap = None
    generatedValuesOfSecondaryObjects: GeneratedValuesOfSecondaryObjectsMap = None


@dataclass
class GetChildrenInputData(TcBaseObj):
    """
    This structure defines the object for which to retrieve children and optionally the propertyNames from which to
    retrieve those children.  If propertyNames is undefined, then the Teamcenter <Type>_DefaultChildProperties
    preference is used.
    
    :var clientId: Identifier used to track the input object.  Primarily this is used to identify which PartialError
    corresponds to which input object.
    :var obj: The object for which to retrieve children.
    :var propertyNames: The properties to use to retrieve children.  Only the properties defined in this list are used.
    If this list is empty, then the children are returned based on the <Type>_DefaultChildProperties and
    <Type>_PseudoFolders preferences.  Please see the Preferences and Environment Variables Reference and the
    RichClient Interface Guide for information on configuring there preferences
    """
    clientId: str = ''
    obj: BusinessObject = None
    propertyNames: List[str] = ()


@dataclass
class GetChildrenOutput(TcBaseObj):
    """
    This structure contains the children output for the defined property name.
    
    :var children: This is the list of Teamcenter business object children which exist within this property name.
    :var propertyName: The property the child belongs to in the parent input object.
    """
    children: List[BusinessObject] = ()
    propertyName: str = ''


@dataclass
class GetChildrenResponse(TcBaseObj):
    """
    Output structure for the getChildren operation.
    
    :var serviceData: The service data contains any partial errors which may have been encountered during processing. 
    All business objects which are returned are added to the serviceData's plain objects list.
    :var objectWithChildren: A map of requested objects and a list of children(business
    object/vector<GetChildrenOutput>).
    """
    serviceData: ServiceData = None
    objectWithChildren: GetChildrenOutputMap = None


@dataclass
class GetPasteRelationsInputData(TcBaseObj):
    """
    Object input structure of getPasteRelations operation.
    
    :var clientId: Identifier used to track the input object.  Primarily this is used to identify which PartialError
    corresponds to which input object.
    :var obj: The parent object for which to get the paste relations.
    :var childTypeName: The child type name for which the paste relations will be retrieved.
    """
    clientId: str = ''
    obj: BusinessObject = None
    childTypeName: str = ''


@dataclass
class GetPasteRelationsOutput(TcBaseObj):
    """
    Output structure of getPasteRelations operation.  This structure contains all the paste relations for a given
    business object.
    
    :var pasteRelationsInfo: This is the list of paste relation names and other info for the given business object.
    :var indexOfPreferred: The zero-based index of the preferred paste relation in the list of paste relation names.
    """
    pasteRelationsInfo: List[PasteRelationsInfo] = ()
    indexOfPreferred: int = 0


@dataclass
class GetPasteRelationsResponse(TcBaseObj):
    """
    Response of getPasteRelations operation.  This structure contains the servicedata and the outputMap to look up the
    paste relations for a given business object.
    
    :var serviceData: The service data contains any partial errors which may have been encountered during processing. 
    All business objects which are returned are added to the serviceData&apos;s plain objects list.
    :var outputMap: A map of input parent object to a vector of GetPasteRelationsOutput objects.
    """
    serviceData: ServiceData = None
    outputMap: GetPasteRelationsOutputMap = None


@dataclass
class PasteRelationsInfo(TcBaseObj):
    """
    Object output structure of getPasteRelations operation.
    
    :var pasteRelationName: The paste relation name.
    :var isExpandable: A flag indicating whether the relation will enable the children to be shown under the parent
    when expanded.
    """
    pasteRelationName: str = ''
    isExpandable: bool = False


@dataclass
class ReviseIn(TcBaseObj):
    """
    Input structure for 'reviseObjects' operation of any revisable object.
    
    :var targetObject: Target object being revised 
    :var reviseInputs: Map of property name(key) and property values(values) in string format, to be set on new object
    being created with revise. The calling client is responsible for converting the different property types (int,
    float, date .etc) to a string using the appropriate toXXXString functions in the SOA client framework Property
    class.
    :var deepCopyDatas: Relevant information regarding applicable deep copy rules for the objects attached to the
    targetobject.
    """
    targetObject: BusinessObject = None
    reviseInputs: PropertyValues = None
    deepCopyDatas: List[DeepCopyData] = ()


@dataclass
class ReviseObjectsResponse(TcBaseObj):
    """
    Output structure of 'reviseObjects' operation have information of new resultant objects created and error
    information if any.
    
    :var output: The target object and the newly created revised objects.
    :var reviseTrees: List corresponding to the input target objects that holds mapping between the original objects
    and the copied objects.
    :var serviceData: Data containing created objects, errors, etc.
    """
    output: List[ReviseOut] = ()
    reviseTrees: List[ReviseTree] = ()
    serviceData: ServiceData = None


@dataclass
class ReviseOut(TcBaseObj):
    """
    Member of Output structure of 'reviseObjects' operation, having information of original and newly created objects.
    
    :var targetObject: Original object being revised.
    :var objects: List of newly created objects during revise operation.
    """
    targetObject: BusinessObject = None
    objects: List[BusinessObject] = ()


@dataclass
class ReviseTree(TcBaseObj):
    """
    Member of Output structure of 'reviseObjects' operation, having information of attached objects and newly created
    objects from them.
    
    :var originalObject: Original object.
    :var objectCopy: Object copy after Revise. This could be NULL if no copy was made or same as original object if the
    copy is a reference to the original.
    :var childReviseNodes: List of child revise nodes for next level of the tree.
    """
    originalObject: BusinessObject = None
    objectCopy: BusinessObject = None
    childReviseNodes: List[ReviseTree] = ()


@dataclass
class SubTypeNamesInput(TcBaseObj):
    """
    The parent business object type name and context for which the sub business object type names are to be retrieved.
    
    :var typeName: The business object type name for which sub business object type names are to be returned. If all
    sub business object type names are needed then pass BusinessObject as input.
    :var contextName: Context name based on which server returns the business object type names.
    
    Supported contexts:
    
    subtypes: Returns all sub business object type names.
    :var exclusionPreference: Name of the preference to be used to exclude the sub business object type names from the
    output. Preference needs to be a multi-valued to specify the names of the business object types to be excluded.
    
    This is optional. It could be empty string.
    """
    typeName: str = ''
    contextName: str = ''
    exclusionPreference: str = ''


@dataclass
class SubTypeNamesOut(TcBaseObj):
    """
    This is the output structure which holds list of sub business object type names for a given business object type
    based on the context specified.
    
    :var typeName: The parent business object type name.
    :var contextName: Name of the context for which  the typeName represents.
    :var exclusionPreference: Name of the preference to be used to exclude the sub business object type names from the
    output. Preference needs to be a multi-valued to specify the names of the business object types to be excluded.
    
    This is optional. It could be empty string.
    :var subTypeNames: List of sub business object type names based on the context name.
    :var displayableSubTypeNames: List of sub business object type displayable names based on the context name.
    """
    typeName: str = ''
    contextName: str = ''
    exclusionPreference: str = ''
    subTypeNames: List[str] = ()
    displayableSubTypeNames: List[str] = ()


@dataclass
class SubTypeNamesResponse(TcBaseObj):
    """
    The returned business object type names based on the context for each input business object type.
    
    :var output: List of  business object type names.
    :var serviceData: The Service data.
    """
    output: List[SubTypeNamesOut] = ()
    serviceData: ServiceData = None


@dataclass
class ValidateInput(TcBaseObj):
    """
    This structure contains the necessary input for the 'validateValues' operation to support generic property
    validation while retaining support for some existing user exits for specific properties.  The 'validateValues'
    operation details legacy user exit support.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.
    :var operationType: The operation type for which the input values are to be validated. Valid values are:
    
    - 0 for create
    - 1 for revise
    - 2 for saveas
    - 3 for generic validation
    
    
    :var businessObjectName: The name of the business object for which the property values are to be validated. 
    :var propValuesMap: A map where the key (string) is a property name and the value (string) is the property value to
    be validated.  The value input is a list to support multi-value properties. The values of the properties are to be
    provided as strings.
    :var compoundObjectInput: A map where the key (string) is a property name and the value ('ValidateInput') is input
    for the property. This contains the property values of the secondary objects.
    """
    clientId: str = ''
    operationType: int = 0
    businessObjectName: str = ''
    propValuesMap: PropertyValues = None
    compoundObjectInput: CompoundValidateInputMap = None


@dataclass
class ValidateValuesResponse(TcBaseObj):
    """
    The response from the 'validateValues' operation.
    
    :var validationResults: A map where the key (string) is the 'clientId' and the value ('ValidationResults') is a
    list of validation results for the properties.
    :var serviceData: Service data containing errors that occurred during the operation.
    """
    validationResults: ValidationResultsMap = None
    serviceData: ServiceData = None


@dataclass
class ValidationResults(TcBaseObj):
    """
    This structure holds the property validation results for 'validateValues' operation.
    
    :var uniqueID: If true, the MFK value is unique.  If false, the MFK value is not unique.
    :var validationStatus: A list of ValidationStatus, which contains the validation status for each input property
    value.
    """
    uniqueID: bool = False
    validationStatus: List[ValidationStatus] = ()


@dataclass
class ValidationStatus(TcBaseObj):
    """
    This structure holds the validation status for a property.
    
    :var propName: The property name.
    :var valueStatus: The status of the property value validation. The possible values are:
    
    - 0 for valid - The input value is valid for use as-is.
    - 1 for invalid - The input value is not valid and cannot be used.
    - 2 for override - The input value cannot be used, but should be replaced by the value in the modifiedValue field.
    - 3 for duplicate - The input value is not valid because it is considered a duplicate and cannot be used.
    
    
    :var modifiedValue: The modified property value if the input value is modified by the user exit or by naming rule
    validation according to naming rule pattern and case specification. The modified property value should be used for
    subsequent create, saveAs and revise operation. It will hold value if property value validation status is override
    i.e. valueStatus = 2 otherwise it will be empty.
    This is a list to support multi-value properties.
    """
    propName: str = ''
    valueStatus: int = 0
    modifiedValue: List[str] = ()


@dataclass
class DeepCopyData(TcBaseObj):
    """
    The 'DeepCopyData' data structure holds the relevant information regarding applicable deep copy rules.
    
    :var attachedObject: Other side object.
    :var propertyName: Name of relation type or reference property for which DeepCopy rule is configured.
    :var propertyType: If 'Relation', it represents Relation type property. If 'Reference', it represents Reference
    property.
    :var copyAction: DeepCopy action [NoCopy, CopyAsReference, CopyAsObject or Select].
    :var isTargetPrimary: If true the target object is processed as primary, otherwise it is processed as a secondary
    object.
    :var isRequired: If true, the copy action can not be modified. If false, the copy action can be changed different
    action by the user. In this case, the copy action field in the revise dialog is editable.
    :var copyRelations: If true, the custom properties on the source relation object are copied over to the
    newly-created relation. If false, custom properties are not copied.
    :var operationInputTypeName: OperationInput type name.
    :var childDeepCopyData: List of DeepCopy data for the secondary objects on the other side.
    :var operationInputs: OperationInput field to capture property values of attached object, to apply on copied object
    of attached object. Map of property name(key) and property values(values) in string format of attached object, to
    be set on copied object of attached object. The calling client is responsible for converting the different property
    types (int, float, date .etc) to a string using the appropriate toXXXString functions in the SOA client  framework
    Property class.
    """
    attachedObject: BusinessObject = None
    propertyName: str = ''
    propertyType: str = ''
    copyAction: str = ''
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False
    operationInputTypeName: str = ''
    childDeepCopyData: List[DeepCopyData] = ()
    operationInputs: PropertyValues = None


"""
A map of property name and GeneratedValue list.
"""
GeneratedValuesMap = Dict[str, GeneratedValue]


"""
A map of property name and list of  GeneratedValues.
"""
GeneratedValuesOfSecondaryObjectsMap = Dict[str, List[GeneratedValuesOutput]]


"""
This is a map of input BusinessObject to a vector of GetChildrenOutput objects.  It is used to look up the output for a given input BusinessObject.
"""
GetChildrenOutputMap = Dict[BusinessObject, List[GetChildrenOutput]]


"""
This is a map of input to a vector of GetPasteRelationOutput objects. It is used to look up the output for a given input business object.
"""
GetPasteRelationsOutputMap = Dict[BusinessObject, GetPasteRelationsOutput]


"""
Additional parameters needed to generate the next values.
"""
AdditionalInputParamsMap = Dict[str, str]


"""
A map of property name and values list.
"""
PropValuesMap = Dict[str, List[str]]


"""
A map of  property name and  naming rule pattern  pairs.
"""
PropertyNameWithSelectedPatternMap = Dict[str, str]


"""
Map of property name(key) and property values(values) in string format.
"""
PropertyValues = Dict[str, List[str]]


"""
A map where the key is the 'clientId' and the value is a list of 'ValidationResults' for the properties.
"""
ValidationResultsMap = Dict[str, ValidationResults]


"""
A map of property name and PropertyValuesInput list.
"""
CompoundObjectInputMap = Dict[str, List[GenerateNextValuesIn]]


"""
A map where the key is the property name and the value is a list of 'ValidateInput'.  This map contains the property values of the secondary objects.
"""
CompoundValidateInputMap = Dict[str, List[ValidateInput]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2015_10.DataManagement import DefaultRelationInfo
from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2012_10.DataManagement import ObjectLsdInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDeclarativeStyleSheetResponse(TcBaseObj):
    """
    Response sent to client from the getDeclarativeStyleSheets operation.
    
    :var declarativeUIDefs: Declarative UI Style Sheet Output.
    :var serviceData: Service Data.
    """
    declarativeUIDefs: List[DeclarativeUIDef] = ()
    serviceData: ServiceData = None


@dataclass
class GetDefaultRelationIn(TcBaseObj):
    """
    Holds information of the primary and secondary object types.
    
    :var primaryType: String literal representing the type of primary object.
    :var secondaryType: String literal representing the type of secondary object.
    """
    primaryType: str = ''
    secondaryType: str = ''


@dataclass
class GetDefaultRelationInfo(TcBaseObj):
    """
    Contains information of primary and secondary type and default relation between them. In case default relation
    between primary and secondary type is not found, then the DefaultRelationInfo object will be empty.
    
    :var primaryType: String literal representing the type of primary object.
    :var secondaryType: String literal representing the type of secondary object.
    :var defaultRelation: Contains ImanType, internal name and display name information of default relation between
    primary and secondary type.
    """
    primaryType: str = ''
    secondaryType: str = ''
    defaultRelation: DefaultRelationInfo = None


@dataclass
class GetDefaultRelationResponse(TcBaseObj):
    """
    Response from getDefaultRelation operation.
    
    :var output: List of GetDefaultRelationInfo objects.
    :var serviceData: The SOA service data.
    """
    output: List[GetDefaultRelationInfo] = ()
    serviceData: ServiceData = None


@dataclass
class NameValue(TcBaseObj):
    """
    This structure hold property name and list of property values. This is a generic container that contains the
    property name as string and the value is the string representation of the property value.
    
    :var name: Property name string that needs to be modified.
    :var values: A list of property values that will be used to update the input object with corresponding values. This
    list contains string representation of property value. The calling client is responsible for converting the
    different property types (int, float, date .etc) to a string using the appropriate to XXXString functions in the
    SOA client framework's Property class.
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class PropertyNameValue(TcBaseObj):
    """
    Input Structure containing information about what properties are to be populated. 
    For normal (non-Table )Properties, user has to populate values. 
    For Table Properties, user has to fill the rowValues structure
    
    :var name: Property name that needs to be modified.
    :var values: Property values to update object with. This list contains string representation of the property value.
    The calling client is responsible for converting the different property types (int, float, date .etc) to a string
    using the appropriate toXXXString functions in the SOA client framework's Property class.
    :var rowValues: A list of child object structures. Child structures are populated in case the property contains a
    collection of sub objects. E.g. table properties. For other properties, the list needs to be empty.
    """
    name: str = ''
    values: List[str] = ()
    rowValues: List[RowData] = ()


@dataclass
class RowData(TcBaseObj):
    """
    A structure to hold child information associated with the main object.
    It contains uid of the child, NameValue type of row and its name value pairs for different properties.
    
    :var uid: The child object UID. If the UID is empty or a NULL UID, a new object is created with details as
    populated in nameValues.
    :var rowType: The subtype of Fnd0NameValue business object. Valid value can only be one from below mentioned list
    of values :
    Fnd0NameValueDate,
    Fnd0NameValueDouble,
    Fnd0NameValueInt,
    Fnd0NameValueLogical, 
    Fnd0NameValueString.
    :var nameValues: List of named property values to be saved. If UID is not blank then the properties populated in
    this structure will be updated. If UID is blank, then a new child object is created based on the name values input.
    """
    uid: str = ''
    rowType: str = ''
    nameValues: List[NameValue] = ()


@dataclass
class SaveEditAndSubmitInfo(TcBaseObj):
    """
    Structure represents the parameters required to save the edits on the input objects and submit them to a workflow
    process.
    
    :var tcobject: Teamcenter object (POM_object) to save edit and submit to a workflow process.
    :var propertyNameValues: Property name and values structure that will contain all property names and corresponding
    values that needs to be saved.
    :var objLsds: The last set date information for the object.
    :var isPessimisticLock: The flag to control whether this method call performs optimistic locking or not.
    If false, operation takes optimistic lock on the object. In optimistic locking the object is locked (based on the
    object last saved date) before performing the actual save and lock is released after save is complete.
    If true, object needs to be checked out before making any edits (pessimistic locking). The locking of object is
    done during loadDataForEditing operation.
    :var workflowData: The workflow information that the input object needs to submit to a workflow. Workflow
    information is stored in a name and value string map (string/list of strings). If workflowData map is empty,
    objects will not be submitted to any workflow.
    Supported keys:
    
    &bull; submitToWorkflow: Boolean Property to define that input object need to submit to workflow process or not. It
    can contain true or false as value. 
    &bull; processName: Process name string. 
    &bull; processDescription: Process description string. 
    &bull; processTemplate: Name of the process template to be used to create new workflow process. 
    &bull; processAssignmentList: Name of the process assignment list to use while creating new workflow process. 
    
    If property submitToWorkflow value is true and processTemplate value is empty, then workflow process template to be
    used for workflow creation should be specified in the preference (<TypeName>_default_workflow_template) defined for
    the submitted object type.
    """
    tcobject: BusinessObject = None
    propertyNameValues: List[PropertyNameValue] = ()
    objLsds: List[ObjectLsdInfo] = ()
    isPessimisticLock: bool = False
    workflowData: PropertyValues2 = None


@dataclass
class DeclarativeUIDef(TcBaseObj):
    """
    The declarative UI definition. Which can be used to define a particular part of web page with the support of
    declarative UI framework.
    
    :var view: The declarative view definition. It consists of HTML custom elements.
    :var viewModel: The view model of declarative UI in JSON format. It can be bound to declarative view.
    :var glueCodeJS: The piece of JavaScript code to glue the declarative view and view model. JavaScript functions can
    be defined in glue code and be referenced in the view model. The functions which are referenced will be called by
    declarative UI framework, and the return values are consumed to build up the view model.
    """
    view: str = ''
    viewModel: str = ''
    glueCodeJS: str = ''


"""
Map (string, list of strings) that is a generic container that represents property values. The key is the property name and the value is the string representation of the property value.
"""
PropertyValues2 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0Client, Fnd0CommandCollection, Fnd0AbstractCommand, Fnd0ClientScope, Fnd0UIConfigCollectionRel
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetUIConfigInput(TcBaseObj):
    """
    Contains input information requiredto retrieve UI configurations from the Teamcenter database.
    
    :var clientScopeURIs: List of client scope URIs representing, for example location.sublocation in Active Workspace.
    (Fnd0ClientScope::fnd0ClientScopeURI). If empty, the UI Configuration for all client scopes are returned.
    :var scope: The scope of the desired UI configuration information. This includes the name of the scope (i.e. a
    user, group or role) and scope query parameter information.
    :var client: Client information including client name and hosting client name.
    """
    clientScopeURIs: List[str] = ()
    scope: ScopeInput = None
    client: ClientInput = None


@dataclass
class GetUIConfigResponse(TcBaseObj):
    """
    This structure returns information to the client about column configuration and command applicability. The
    ServiceData contains information about errors encountered during processing.
    
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    :var uiConfigInfo: List of configuration information including command and column information.
    """
    serviceData: ServiceData = None
    uiConfigInfo: List[ClientConfigurations] = ()


@dataclass
class ClientConfigurations(TcBaseObj):
    """
    This structure contains command and column configuration data for the indicated client.
    
    :var client: The client for which the configurations are applicable.
    :var columnConfigurations: List of column configurations.
    :var commandConfigurations: List of command configurations.
    """
    client: str = ''
    columnConfigurations: List[ColumnConfigData] = ()
    commandConfigurations: List[CommandConfigData] = ()


@dataclass
class ClientInput(TcBaseObj):
    """
    Specifies client input information including client name and hosting client name.
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property.
    :var hostingClientName: Specifies the name of a hosting Client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases. This value must match a value of the fnd0ClientName property. For example,
    if Client A is integrated with Client B and the user can invoke Client B commands from within Client A, the input
    to getUiConfigs would specify Client A as hosting Client and Client B as the Client. If the caller wanted native
    commands for Client A, Client A would be specified as Client and hosting Client would be empty.
    """
    clientName: str = ''
    hostingClientName: str = ''


@dataclass
class ColumnConfig(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information, and the default sort direction.
    
    :var columnConfigId: The unique identifier of the column.
    :var sortDirection: How the columns are sorted. Valid values are Ascending and Descending.
    :var columns: Ordered list of column details.
    """
    columnConfigId: str = ''
    sortDirection: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class ScopeInput(TcBaseObj):
    """
    Contains scope input information including scope name and scope query parameter.
    
    :var scopeName: The name of a scope. For Site and current login user, this value should be empty.
    :var scopeQueryParam: The query scope that is used to retrieve the UI configurations. Valid values are Site, Group,
    Role, User, LoginUser, and AvailableForLoginUser. Site returns the configuration defined for the site. Group
    returns the configuration defined for a specific Teamcenter Group. Role returns the configuration defined for a
    specific Teamcenter Role. User returns the configurations defined for a specific Teamcenter User. LoginUser returns
    the current configuration defined for the current login user. AvailableForLoginUser returns all the configurations
    available for the current login user. The value for scopeName should be empty when scopeQueryParam is set to:
    "Site", "LoginUser" or "AvailableForLoginUser".
    """
    scopeName: str = ''
    scopeQueryParam: str = ''


@dataclass
class ColumnConfigData(TcBaseObj):
    """
    This structure returns information about the column configuration definitions for a scope, hosting client and
    client scope.
    
    :var scope: The scope for which the column data is applicable.
    :var hostingClient: The name of hosting client for which the list of column configurations is applicable. This
    value must correspond to the value of the property fnd0ClientName for an Fnd0Client object in the Teamcenter
    database.
    :var clientScopeURI: The client scope for which the list of column configurations is applicable. This must match a
    value of fnd0ClientScopeURI which is the unique identifier of a client scope (Fnd0ClientScope).
    :var columnConfigurations: List of column configuration details.
    """
    scope: str = ''
    hostingClient: str = ''
    clientScopeURI: str = ''
    columnConfigurations: List[ColumnConfig] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client and pixel width.
    
    :var typeName: The Business Object type for the value displayed in the column. This can be any valid Teamcenter
    business object type.
    :var propertyName: The property name for the value displayed in the column.
    :var sortByFlag: True if the column is used to sort the information displayed to the user.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between1 and 500.
    """
    typeName: str = ''
    propertyName: str = ''
    sortByFlag: bool = False
    pixelWidth: int = 0


@dataclass
class CommandCollectionInfo(TcBaseObj):
    """
    Contains a command collection and indexes to its children commands or command collections.
    
    :var childIsCollection: If true ,the child is a command collection; otherwise, the child is a command.
    :var childIsVisible: If true, command or collection is visible; otherwise, the command or collection is not visible.
    :var childConfigIndex: Index of the child in either the CommandConfigData:commands list or the
    CommandConfigData:cmdsCollections list. It is used by client to build the command hierarchy.
    :var commandCollection: Fnd0CommandCollection object.
    """
    childIsCollection: List[bool] = ()
    childIsVisible: List[bool] = ()
    childConfigIndex: List[int] = ()
    commandCollection: Fnd0CommandCollection = None


@dataclass
class CommandConfigData(TcBaseObj):
    """
    This structure returns information about the command configuration definitions for a client scope URI.
    
    :var scopeName: The scope for which the command data is applicable.
    :var hostingClient: The hosting client for which the command data is applicable.
    :var clientScope: The client scope URI containing the list of command configurations.
    :var cmdCollections: List of all top level and child command collections in the client scope.
    :var cmdCollectionIndex: Index into cmdCollections list for top level command collections in the client scope.
    :var commands: List of all command children accessible in the client scope.
    :var commandCollectionRels: List of all commandCollectionRel objects that associate top level command collections
    with client scope.
    """
    scopeName: str = ''
    hostingClient: Fnd0Client = None
    clientScope: Fnd0ClientScope = None
    cmdCollections: List[CommandCollectionInfo] = ()
    cmdCollectionIndex: List[int] = ()
    commands: List[Fnd0AbstractCommand] = ()
    commandCollectionRels: List[Fnd0UIConfigCollectionRel] = ()



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetIndexedObjectsAndUpdateResponse(TcBaseObj):
    """
    Structure containing list of indexed objects UIDs and service data with partial errors.
    
    :var indexedObjectsUIDs: The list of indexed objects UIDs
    :var serviceData: The service data object
    """
    indexedObjectsUIDs: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class QueryAndUpdateDataInput(TcBaseObj):
    """
    Input structure containing option value to query and update the objects among newly added, modified, deleted or
    all. It also includes a boolean dryRun flag.
    
    The queryAndUpdateOptions value is passed to skip or process in finding and updating the newly added, modified,
    deleted objects. When dryRun flag is true, only queries will be executed and no updates to the records are made.
    This dryRun mode can be used to know the statistics about the objects that are awating sync.
    
    :var queryAndUpdateOptions: Bit values. Supported values are Add=1, Modified=2, Deleted=4
    :var dryRun: If true, in dry run mode Queries will be executed; however, no updates to records are made.
    """
    queryAndUpdateOptions: int = 0
    dryRun: bool = False


@dataclass
class QueryAndUpdateSyncDataResponse(TcBaseObj):
    """
    Response structure contains the information about what the operation has found. The partial errors are added to the
    service data.
    
    :var syncInfo: A map(string, list of strings) containing status counts or other  information that can be used for
    further processing.
    :var serviceData: The service data object.
    """
    syncInfo: SyncInfo = None
    serviceData: ServiceData = None


"""
A map containing information generated during the operation execution.
This includes information like count of objects found as newly added or modified or deleted that need to be synced. It can also contain other information like logfile names etc... needed by the caller.

The syncInfo will contain "STATUSCOUNT" as key and value is a list of strings as mentioned below with the counts of the objects found for the respective queries. Additional key/value pairs can be supported in future.
ADDED:<Count>
DELETED:<Count>
MODIFIED:<Count>
MODIFIED VIA RELATION:<Count>
"""
SyncInfo = Dict[str, List[str]]



from __future__ import annotations

from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class OverlapStateList(TcBaseObj):
    """
    A list of indicators for the degree of overlap between two effectivity expressions.
    
    :var overlapStates: A vector of overlap enumeration values. Valid values for an overlap enumeration are:
    -    OverlapStateNone : The two expressions have no overlap. There is no satisfing solution common to both
    expressions. A conjunction (AND combination) of the two is unsatisfiable.
    -    OverlapStateSubset : The two expressions overlap. The expression's solution set is a subset of the reference
    expression's solution set. The conjunction (AND combination) of the expression with the negated reference
    expression is unsatisfiable.
    -    OverlapStateMatch : The two expressions are logically equivalent. Every solution that satifies one expression
    also satifies the other, and vice versa.
    -    OverlapStateSuperset : The two expressions overlap. The expression's solution set is a superset of the
    reference expression's solution set. The conjunction (AND combination) of the negated expression with the reference
    expression is unsatisfiable.
    -    OverlapStateIntersect : The two expressions overlap. The expression's solution set has some overlap with the
    refernce expression's solution set.
    
    """
    overlapStates: List[OverlapState] = ()


@dataclass
class EffectivityDisplayStringResponse(TcBaseObj):
    """
    A list of localized string representations for effectivity expressions. The format is a pure output format intended
    to increase ease of reading. It cannot be used as input because its format allows ambiguous syntax.
    
    :var displayString: Vector of effectivity display strings. This format is the same as for effectivity formula
    properties mdl0effectivity_formula and mdl0allowed_eff_formula.
    :var serviceData: Teamcenter service data.
    """
    displayString: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class EffectivityOverlapStateResponse(TcBaseObj):
    """
    Indicates the degreee of overlap for pairs of effectivity expressions.
    
    :var overlapStates: A vector of overlap enumeration values.
    :var serviceData: Teamcenter service data.
    """
    overlapStates: List[OverlapStateList] = ()
    serviceData: ServiceData = None


class OverlapState(Enum):
    """
    Operlap states enumerations.
    
    :var OverlapStateNone: The two expressions have no overlap. There is no satisfing solution common to both
    expressions. A conjunction (AND combination) of the two is unsatisfiable.
    :var OverlapStateSubset: The two expressions overlap. The expression's solution set is a subset of the reference
    expression's solution set. The conjunction (AND combination) of the expression with the negated reference
    expression is unsatisfiable.
    :var OverlapStateMatch: The two expressions are logically equivalent. Every solution that satifies one expression
    also satifies the other, and vice versa.
    :var OverlapStateSuperset: The two expressions overlap. The expression's solution set is a superset of the refernce
    expression's solution set. The conjunction (AND combination) of the negated expression with the reference
    expression is unsatisfiable.
    :var OverlapStateIntersect: The two expressions overlap. The expression's solution set has some overlap with the
    refernce expression's solution set.
    """
    None_ = 'OverlapStateNone'
    Subset = 'OverlapStateSubset'
    Match = 'OverlapStateMatch'
    Superset = 'OverlapStateSuperset'
    Intersect = 'OverlapStateIntersect'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import AppInterface
from tcsoa.gen.Ai._2008_06.Ai import GenerateScopedMultipleStructure2Response, GenerateScopedSyncRequest2Response, ObjectsWithConfig, ConfigurationContextPair, CompareConfigurationContextsResponse
from typing import List
from tcsoa.gen.Ai._2006_03.Ai import RequestDetail
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def compareConfigurationContexts(cls, configurationsToCompare: List[ConfigurationContextPair]) -> CompareConfigurationContextsResponse:
        """
        compareConfigurationContexts: allows clients to check if configurationcontext
        with different uids are equivalent or not. This is because a configurationcontext
        is used to capture runtime revisionrules and there might be 2 configurationcontext
        created with same set of runtime rules. This interface is for backward
        compatibility with Ai. configurationsToCompare pairs of configurationContexts to compare with one other.
        """
        return cls.execute_soa_method(
            method_name='compareConfigurationContexts',
            library='Ai',
            service_date='2008_06',
            service_name='Ai',
            params={'configurationsToCompare': configurationsToCompare},
            response_cls=CompareConfigurationContextsResponse,
        )

    @classmethod
    def generateScopedMultipleStructure2(cls, aiObject: AppInterface, objectsToProcess: List[ObjectsWithConfig], exportTransferMode: str, serverMode: int) -> GenerateScopedMultipleStructure2Response:
        """
        GenerateScopedMultiple2Structure: Same as GenerateScopedSyncRequest2 - except no aiObject is needed. If
        specified - it is only
        used to get the TransferMode (in case the exportTransferMode is not specified).
        objects or occurrence group objects - specified as application refs. The configuration is optional
        if the ids consist of StructureContexts. ApplicationRefs can be ids of occurrence from a previous
        export from TC, or APNs or AbsOccs, or OccurrenceGroup or an Item/Itemrev(only one in that last case).
        If the Appref is custom (non TcEng AppRef), occurrence appref must resolve to AbsOccurrence or APN ),
        or they can be ids of structure context/occurrence group objects. The return is the transient file ticket
        for the plmxml file generated. In case any of the input apprefs cannot be processed they will be returned
        in the data member of response. Errors during plmxml processing will be returned in partialerror as
        xml string, based on TcError.xsd in iman_data folder. Configuration structure can be used to specify
        default revrule (if true) all other fields are ignored. Basically, they are declared in the order of
        precedence (where duplication is possible).
        """
        return cls.execute_soa_method(
            method_name='generateScopedMultipleStructure2',
            library='Ai',
            service_date='2008_06',
            service_name='Ai',
            params={'aiObject': aiObject, 'objectsToProcess': objectsToProcess, 'exportTransferMode': exportTransferMode, 'serverMode': serverMode},
            response_cls=GenerateScopedMultipleStructure2Response,
        )

    @classmethod
    def generateScopedSyncRequest2(cls, aiObject: AppInterface, objectsToProcess: List[ObjectsWithConfig], requestDetail: RequestDetail) -> GenerateScopedSyncRequest2Response:
        """
        generateScopedSyncRequest2: Generates a new Sync Request for the given occurrences (from any context) or
        Structure Context
        objects or occurrence group objects - specified as application refs. The configuration is optional
        if the ids consist of StructureContexts. ApplicationRefs can be ids of occurrence from a previous
        export from TC, or APNs or AbsOccs, or OccurrenceGroup or an Item/Itemrev(only one in that last case).
        If the Appref is custom (non TcEng AppRef), occurrence appref must resolve to AbsOccurrence or APN ),
        or they can be ids of structure context/occurrence group objects. The return will be the details
        of the newly created Sync Request. Note that the name, desc, scope, updateType of this request are
        based on the passed in requestDetail. The other fields of the RequestDetail are
        not used during input. In case any of the input apprefs cannot be processed they will be returned
        in the failedIndices structure. Errors during plmxml processing will be returned in partialerror
        , based on TcError.xsd in iman_data folder. This differs from generateScopedSyncRequest, in that it
        allows you to specify the reference structures for process.
        """
        return cls.execute_soa_method(
            method_name='generateScopedSyncRequest2',
            library='Ai',
            service_date='2008_06',
            service_name='Ai',
            params={'aiObject': aiObject, 'objectsToProcess': objectsToProcess, 'requestDetail': requestDetail},
            response_cls=GenerateScopedSyncRequest2Response,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.ProjectManagement._2008_06.ScheduleManagement import StringValContainer, TypedAttributeContainer
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenericAttributesContainer(TcBaseObj):
    """
    A collection of KeyValContainers or StringValContainers reprenting the create or updates for a single object, such
    as Schedule, ScheduleTask, TaskDependency, or ResourceAssignment
    
    :var serviceType: The service type. Valid values are "GET" or "POST","VALIDATE" or "PERSIST"
    :var operationType: The operation type (may be set by the system if the API does not have multiple contracts)
    :var scheduleUid: the schedule uid. This is required for batch updates in multiple schedules.
    :var object: The tag to the object being updated
    :var stringValContainer: A collection of updates. Valid values are  key = property name which is to be updated eg
    "priority"
     value = value of the property eg. "2"
    :var typedAttributesContainer: A collection of updates for extended properties if there are any (optional)
    """
    serviceType: ServiceAction = None
    operationType: TranslatorOperation = None
    scheduleUid: str = ''
    object: BusinessObject = None
    stringValContainer: List[StringValContainer] = ()
    typedAttributesContainer: List[TypedAttributeContainer] = ()


@dataclass
class GenericResponseContainer(TcBaseObj):
    """
    This is the generic response container
    
    :var serviceData: The ServiceData
    :var objectUids: The list of objects uid
    :var objects: The list of objects
    """
    serviceData: ServiceData = None
    objectUids: List[str] = ()
    objects: List[BusinessObject] = ()


@dataclass
class ScheduleModifyContainer(TcBaseObj):
    """
    The information necessary to update a schedule.  It contains information about new, updated, and deleted tasks,
    resource assignments, and task dependencies.
    
    :var scheduleUid: the schedule uid
    :var serviceType: The service action
    :var deletedTasks: The deleted tasks
    :var deletedDependencies: The deleted dependencies
    :var deletedAssignments: The deleted assignments
    :var scheduleUpdates: the schedule updates
    :var deleteAllTasks: Delete all tasks flag
    :var newTasks: The new tasks to add
    :var newDependencies: The new dependencies
    :var newAssignments: The new assignments
    :var taskUpdates: The existing task updates
    :var dependenciesUpdate: The existing dependencies updates
    :var assignmentUpdates: The existing assignment updates
    """
    scheduleUid: str = ''
    serviceType: ServiceAction = None
    deletedTasks: List[BusinessObject] = ()
    deletedDependencies: List[BusinessObject] = ()
    deletedAssignments: List[BusinessObject] = ()
    scheduleUpdates: GenericAttributesContainer = None
    deleteAllTasks: bool = False
    newTasks: List[GenericAttributesContainer] = ()
    newDependencies: List[GenericAttributesContainer] = ()
    newAssignments: List[GenericAttributesContainer] = ()
    taskUpdates: List[GenericAttributesContainer] = ()
    dependenciesUpdate: List[GenericAttributesContainer] = ()
    assignmentUpdates: List[GenericAttributesContainer] = ()


class ServiceAction(Enum):
    """
    Enumerator for service actions
    """
    GET = 'GET'
    POST = 'POST'
    VALIDATE = 'VALIDATE'
    PERSIST = 'PERSIST'
    ANALYSE = 'ANALYSE'


class TranslatorOperation(Enum):
    """
    Enumerator for translator operations
    """
    AssignmentAssign = 'AssignmentAssign'
    AssignmentUnassign = 'AssignmentUnassign'
    ScheduleCreate = 'ScheduleCreate'
    ScheduleDelete = 'ScheduleDelete'
    ScheduleRecalculate = 'ScheduleRecalculate'
    ScheduleModify = 'ScheduleModify'
    ScheduleTaskCreate = 'ScheduleTaskCreate'
    ScheduleTaskDelete = 'ScheduleTaskDelete'
    ScheduleTaskUpdate = 'ScheduleTaskUpdate'
    ScheduleGetCriticalPath = 'ScheduleGetCriticalPath'
    ScheduleShift = 'ScheduleShift'
    ScheduleScale = 'ScheduleScale'
    AssignmentUpdate = 'AssignmentUpdate'
    ScheduleTaskMove = 'ScheduleTaskMove'
    Undefined = 'Undefined'
    AssignmentDelegate = 'AssignmentDelegate'
    AssignmentReplace = 'AssignmentReplace'
    AssignmentRevert = 'AssignmentRevert'
    ResourceGetAssignments = 'ResourceGetAssignments'
    DependencyCreate = 'DependencyCreate'
    DependencyUpdate = 'DependencyUpdate'
    DependencyDelete = 'DependencyDelete'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, User, POM_imc
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class CreateRemoteWkfInput(TcBaseObj):
    """
    A structure containing information used for workflow replication (e.g. the name of the process template, workflow
    owner, responsible party, export site, attachments, and attachment relations).
    
    :var processName: Workflow process name.
    :var processDescription: Workflow process description.
    :var attachmentRelationTypes: A list of relation type names to use while attaching the attachments to the workflow
    process (e.g. CMHasProblemItem, CMHasSolutionItem, Fnd0EPMTarget).
    :var processTemplate: Name of the workflow process template used for creating the workflow process. This template
    should exist at the site where the workflow is created.
    :var workflowOwner: Owner of the workflow process.
    :var responsibleParty: Responsible party (Users or ResourcePool) of the root task.
    :var assignedUserList: Users or ResourcePool objects to be used for task assignment.
    :var dueDate: The due date of the source object (e.g. Schedule Task).
    :var sourceObject: Source object (e.g. ScheduleTask).
    :var site: The site where the workflow needs to be created.
    :var attachments: A list of attachments to the workflow process (e.g. ItemRevision, Dataset).
    """
    processName: str = ''
    processDescription: str = ''
    attachmentRelationTypes: List[str] = ()
    processTemplate: str = ''
    workflowOwner: User = None
    responsibleParty: BusinessObject = None
    assignedUserList: List[str] = ()
    dueDate: datetime = None
    sourceObject: BusinessObject = None
    site: POM_imc = None
    attachments: List[BusinessObject] = ()


@dataclass
class CreateWkfInput(TcBaseObj):
    """
    A structure containing the process template name ,workflow process owner, responsible party, attachments, and
    attachment relations. Intent of structure is to hold input data that is required to create a workflow process.
    
    :var processName: Workflow process name.
    :var processDescription: Workflow process description.
    :var processTemplate: Name of the workflow process template used for creating the workflow process.
    :var workflowOwner: User assigned as the owner of the workflow process.
    :var responsibleParty: Responsible party of the workflow process root task. Responsible party can be a User or a
    ResourcePool.
    :var assignedUserList: A list of Users or ResourcePool objects  to be used for task assignment.
    :var dueDate: Task due date.
    :var attachments: A list of attachments to the workflow process (e.g. ItemRevision, Dataset).
    :var attachmentRelationTypes: A list of relation type names to use while attaching the attachment objects to the
    workflow process (e.g. CMHasProblemItem, CMHasSolutionItem, Fnd0EPMTarget).
    """
    processName: str = ''
    processDescription: str = ''
    processTemplate: str = ''
    workflowOwner: User = None
    responsibleParty: BusinessObject = None
    assignedUserList: List[BusinessObject] = ()
    dueDate: datetime = None
    attachments: List[BusinessObject] = ()
    attachmentRelationTypes: List[str] = ()


@dataclass
class CreateWkfOutput(TcBaseObj):
    """
    A structure containing the UID of the workflow root task, a NameValuePairs data type that holds root task property
    names and values, and service data. Intent of structure is to be an output parameter used to hold data pertaining
    to a newly created workflow process following the creation of the workflow process.
    
    :var workflowTask: Workflow root task.
    :var attributes: A map (string/string) of workflow root task property names and values. (e.g. status, process
    instructions, task_result).
    :var serviceData: Service data.
    """
    workflowTask: BusinessObject = None
    attributes: NameValuePairs = None
    serviceData: ServiceData = None


"""
A map (string/string) of attributes names and initial value pairs.
"""
NameValuePairs = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetSubscribablePropertiesResponse(TcBaseObj):
    """
    The response will include a set of subscribable properties for a set of input object types. Partial errors will be
    returned and wrapped in serviceData, if any are specified.
    
    :var criteriaProperties: A list of properties for each input object type
    :var serviceData: Service data with the partial error information
    """
    criteriaProperties: List[CriteriaPropertyStructure] = ()
    serviceData: ServiceData = None


@dataclass
class CriteriaPropertyStructure(TcBaseObj):
    """
    Contains object type and a list of subscribable properties internal names and display name pairs sorted by display
    names.
    
    :var subscribableObjectType: Subscribable object type for which the list of properties are configured.
    :var propInternalNames: List of property internal names of subscribable object types sorted by property&rsquo;s
    display names
    :var propDisplayNames: Sorted list of property display names of subscribable object type
    """
    subscribableObjectType: BusinessObject = None
    propInternalNames: List[str] = ()
    propDisplayNames: List[str] = ()



from tcsoa.gen.Internal_RevRuleMgmt._2019_12.services import RevisionRuleAdministrationService as imp0
from tcsoa.utils import TcService


class RevisionRuleAdministrationService(TcService):
    createOrUpdateRevisionRule = imp0.createOrUpdateRevisionRule
    getRevisionRuleInfo = imp0.getRevisionRuleInfo



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from datetime import datetime


@dataclass
class GetAITypesResponse(TcBaseObj):
    """
    GetAITypesResponse structure contains an additionalInfo element, and serviceData to return partial errors. The
    AdditionalInfo type member currently supports the following:
    
    AdditionalInfo.strMap:
    Key:AITypeNames
    Value: list of AppInterfaceType object names.
    
    AdditionalInfo.objMap:
    Key:Specific AITypeName from the Value above
    Value:AppInterfaceType object object at index 0, Tool optionally at index 1.
    
    For each name in AdditionalInfo.strMap, there is an entry in the AdditionalInfo.objMap. The value being the
    AppInterfaceType object. In case, there is a tool associated with the specific AppInterfaceType object, it will be
    the second index in the list of values.
    
    :var additionalInfo: A generic structure of keys and vectors of values (integer,double,object,string,date types)
    for passing additional metadata. Currently, the following name/value pairs are supported:
    
    strMap entries:
    Key:AITypeNames
    Value: list of AppInterfaceType object names.
    
    For each name in list above, there is an entry in the objMap. The value being the AppInterfaceType object. In case,
    there is a tool associated with the specific AppInterfaceType object, that will be present as the second element in
    this value.
    :var serviceData: Service data capturing partial errors.
    """
    additionalInfo: AdditionalInfo = None
    serviceData: ServiceData = None


@dataclass
class AdditionalInfo(TcBaseObj):
    """
    a generic structure to capture additional information.
    
    :var intMap: A map (string/list of integers) of generic key to integer values.
    :var dblMap: A map (string/list of doubles) of generic key to double values.
    :var strMap: A map (string/list of strings) of generic key to string values.
    :var objMap: A map (string/list of BusinessObjects) of generic key to  BusinessObject values.
    :var dateMap: A map (string/list of dates) of generic key to date values.
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringToStrVectorMap = None
    objMap: StringToObjVectorMap = None
    dateMap: StringToDateVectorMap = None


"""
a map of string to vector of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap = Dict[str, List[BusinessObject]]


"""
A map of string to vector of strings.
"""
StringToStrVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item, BusinessObject, AssemblyArrangement, BOMLine, PSBOMView, ItemRevision, ConfigurationContext
from tcsoa.gen.Cad._2007_01.StructureManagement import RevisionRuleConfigInfo, OccNote, AttributesInfo
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Cad._2009_04.StructureManagement import RelativeStructureParentInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AskChildPathBOMLineOutputNode(TcBaseObj):
    """
    This structure represents a node in the product structure. This structure is tied to a specific client ID of the
    input. Within the context of an input client ID, this structure represents the child path and its corresponding
    BOMLine object in the product structure.
    
    :var childPath: This is the PSOccurrenceThread UID or the Clone Stable ID specified in the input that represents a
    particular node in the context of the child path or occurrence thread chain.
    :var bomline: The BOMLine object corresponding to the input child path.
    """
    childPath: str = ''
    bomline: BOMLine = None


@dataclass
class AskChildPathBOMLinesResponse2(TcBaseObj):
    """
    This structure defines the response from the 'askChildPathBOMLines' operation. The response contains the BOMLine
    objects for each of the input child paths which are still present in the Product Structure in Teamcenter. Each
    BOMLine object in the 'output' is mapped to its corresponding input 'clientId' and one 'childpath'. Only the input
    nodes which are still in the Product Structure in Teamcenter will be in the 'output'. If an input node no longer
    exists in the Product Structure, it will not be present in the 'output'.
    
    :var output: A map of input Client ID to a vector of structure containing the input child path and its
    corresponding BOMLine object.
    :var serviceData: The SOA framework object containing plain objects and error information.
    """
    output: AskChildPathClientIdToBOMLineMap = None
    serviceData: ServiceData = None


@dataclass
class MoveInfo(TcBaseObj):
    """
    This structure represents the information for moving a single occurrence from its current parent assembly to its
    target parent assembly in the context of a higher level assembly.
    
    :var commonParent: The item revision of the common parent assembly where this move is occurring. Only ItemRevision
    type is supported for now.
    :var sourceAssembly: The item revision of the source assembly from where the occurrence is being moved.
    :var occThreadPathToBeMoved: The occurrence thread path of the BOM line to be moved. The occurrence thread path is
    in a bottom up list. The first entry in the list should be occurrence to be moved and the last entry should be the
    occurrence thread for the common parent.
    :var occThreadPathTargetParent: The occurrence thread path of the target parent. The thread path is a bottom up
    list. The first entry in the list is the target parent assembly and the last entry in the list the common parent
    for the move.
    """
    commonParent: BusinessObject = None
    sourceAssembly: BusinessObject = None
    occThreadPathToBeMoved: List[str] = ()
    occThreadPathTargetParent: List[OccThreadEquivalent] = ()


@dataclass
class OccThreadEquivalent(TcBaseObj):
    """
    This structure identifies the child occurrence in the parent assembly. This structure can identify an child
    occurrence that are already existing in Teamcenter as well as the child occurrences that are not yet created in
    Teamcenter.
    
    :var parent: The item revision of the parent assembly. Only item revisions are allowed.
    :var idType: Type of child identifier. The legal values are: 'OccurrenceThread', 'CadOccId', 'ClientId'.
    :var id: The identifier for the occurrence as specified by the 'idType' member. The types of identifiers that can
    be used are the occurrence thread path, CAD occurrence Id and client Id. In cases where an occurrence with the
    specified identifier value does not exist in Teamcenter, the corresponding occurrence will be created.
    :var isNew: flag to indicate if this occurrence is new in the parent assembly.
    """
    parent: BusinessObject = None
    idType: OccThreadIdType = None
    id: str = ''
    isNew: bool = False


@dataclass
class RelOccInfo(TcBaseObj):
    """
    This structure contains the list of attributes information, a flag to specify if the quantity is to be set as
    required, occurrence transform and occurrence notes information for the occurrence.
    The 'MoveInfo' structure contains details on the moving an occurrence from its current parent to a new target
    parent.
    
    :var attrsToSet: Name and value pairs for the attribute information to set or update on the occurrence specified in
    the form of BOM line property names. For example, the BOM line occurrence name property could be specified with the
    attrsToSet name as bl_occurrence_name and the value as the occurrence name.
    :var asRequired: Flag to specify that the quantity is as required. The default value is FALSE.
    :var occTransform: Positioning information for the occurrence. This needs to be ordered in the standard matrix
    format.
    :var occNotes: Note information for the occurrence.
    :var moveInfo: Optional data specifying the occurrence to be moved and the parent to which to be moved to.
    """
    attrsToSet: List[AttributesInfo] = ()
    asRequired: bool = False
    occTransform: List[float] = ()
    occNotes: List[OccNote] = ()
    moveInfo: MoveInfo = None


@dataclass
class RelativeStructureChildInfo3(TcBaseObj):
    """
    This structure contains information for the child node of a relative structure
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure. If clientId is not provided then it can be difficult to align the
    input with the output or any returned errors.
    :var childBomViewTypeName: The name of the BOM view type of the child to be added to the parent BOMViewRevision. If
    not specified, the 'bomViewTypeName' specified in the input preference  'CreateOrUpdateRelativeStructurePref' will
    be used as the default. For example, this can be used in a mult-CAD usage, where a JTView of a child originally
    created in a different CAD system needs to be added to a parent in NX, where the default view type is view.
    :var cadOccId: The CAD occurrence ID or PSOccurrenceThread UID is used to uniquely identify the occurrence under a
    particular context item revision or General Design Element (GDE) for update. The 'cadOccId' can be null for create.
    A valid cadOccId must be passed when this operation is called with the 'RelativeStructureParentInfo' 'complete'
    input set to true. If a valid 'cadOccId' is not supplied when 'complete' is set to true, this operation creates a
    new occurrence and any data associated against an existing occurrence is removed/lost. This parameter depends on
    the 'CreateOrUpdateRelativeStructurePref' 'cadOccIdAttrName' preference for finding the existing BOM line
    :var child: The child object for the occurrence creation. Only ItemRevision and GeneralDesignElement (GDE) are
    supported. If the child occurrence exists, but the input child object is different than the existing child object,
    the existing child will be replaced by the input child. Existing children are found referencing the occurrence
    found by the 'cadOccId' input
    :var occInfo: The property information to set or updated for this occurrence. The optional 'moveInfo' structure
    specifies if this occurrence is being moved to a new parent assembly.
    """
    clientId: str = ''
    childBomViewTypeName: str = ''
    cadOccId: str = ''
    child: BusinessObject = None
    occInfo: RelOccInfo = None


@dataclass
class CreateOrUpdateRelativeStructureInfo4(TcBaseObj):
    """
    Contains the data for creating or updating the relative product structure for a item revision. It includes the
    information about the parent and its object and a list of type 'RelativeStructureChildInfo3' that describes the
    first level children and their occurrence information.
    
    :var parentInfo: Object reference of the context assembly for create or update of the child occurrence, required
    input reference.
    :var childInfo: List of child info structures for creating the occurrences or updating the occurrence attributes.
    If no child objects are specified and 'RelativeStructureParentInfo' 'complete' is true, all existing child objects
    will be removed. If no child objects are specified and 'RelativeStructureParentInfo' 'complete' is false, the input
    is ignored.
    """
    parentInfo: RelativeStructureParentInfo = None
    childInfo: List[RelativeStructureChildInfo3] = ()


@dataclass
class CreateWindowsInfo2(TcBaseObj):
    """
    Main input structure that defines Item or ItemRevision of the top line in the BOMWindow. In the input, either
    revRuleConfigInfo object and objectsForConfigure object(variant rules or saved option set) or configContext object
    is required.
    
    :var clientId: Identifier that helps the client to track the objects created.
    :var item: Item object reference.
    :var itemRev: ItemRevision object reference.
    :var bomView: PSBOMView object reference
    :var revRuleConfigInfo: Structure with information about RevisionRuleConfigInfo
    :var objectsForConfigure: List of variant rules or single stored option set object to set on this window
    :var activeAssemblyArrangement: Active assembly arrangement of this BOMWindow
    :var configContext: ConfigurationContext object reference.
    :var bomWinPropFlagMap: Mapping for window property and respective value that needs to be set on window. User need
    to populate this map with following property string values as key and true or false as value, which will be set or
    unset on the window
    
    Valid property values are 
    
        show_unconfigured_variants
        show_unconfigured_changes
        show_suppressed_occurrences
        is_packed_by_default
        show_out_of_context_lines
        fnd0show_uncnf_occ_eff
        fnd0bw_in_cv_cfg_to_load_md
    """
    clientId: str = ''
    item: Item = None
    itemRev: ItemRevision = None
    bomView: PSBOMView = None
    revRuleConfigInfo: RevisionRuleConfigInfo = None
    objectsForConfigure: List[BusinessObject] = ()
    activeAssemblyArrangement: AssemblyArrangement = None
    configContext: ConfigurationContext = None
    bomWinPropFlagMap: StringMap = None


class OccThreadIdType(Enum):
    """
    Identifies the type of data being sent by the client to identify the child occurrence information in the parent
    assembly.
    The legal values are: 'OccurrenceThread, CadOccId, ClientId.'
    """
    OccurrenceThread = 'OccurrenceThread'
    CadOccId = 'CadOccId'
    ClientId = 'ClientId'


"""
This map contains the return data for the 'AskChildPathBOMLines' operation. The key for the map is the input Client ID. The value will be a vector of 'AskChildPathBOMLineOutputNode'. The output node structure contains an input child path (which can be a PSOccurrenceThread UID or Clone Stable ID) and its corresponding BOMLine object. The map will contain the BOMLine object for all the valid nodes in the input. For nodes that are no longer available in the Product Structure in Teamcenter, there will be no entry in this map. In other words, if particular input has no corresponding BOMLine object for it in the output, it is implied that that node no longer exists in the Product Structure in Teamcenter.
"""
AskChildPathClientIdToBOMLineMap = Dict[str, List[AskChildPathBOMLineOutputNode]]


"""
This is map of string Key to string Value.
"""
StringMap = Dict[str, str]



import os
from dataclasses import dataclass
from typing import List, Dict

from tcsoa.fcc.fcc_client_proxy import FccClientProxy
from tcsoa.gen.Core._2006_03.FileManagement import GetDatasetWriteTicketsInputData, CommitDatasetFileInfo, \
    DatasetFileInfo, DatasetFileTicketInfo
from tcsoa.gen.Core.services import FileManagementService
from tcsoa.gen.Server import ServiceData


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


@dataclass
class PutFileRecord:
    file_info: DatasetFileInfo
    physical_path: str
    ticket_info: DatasetFileTicketInfo = None
    volume_id: str = None


class FileManagementUtility:
    fcc_client_proxy: FccClientProxy = None

    @classmethod
    def ensure_init(cls):
        if cls.fcc_client_proxy is None:
            cls.fcc_client_proxy = FccClientProxy()

    @classmethod
    def put_files(cls, inputs: List[GetDatasetWriteTicketsInputData]):
        cls.ensure_init()
        all_partial_errors = list()
        put_file_recors = dict()
        for i in inputs:
            for dataset_info in i.datasetFileInfos:
                if dataset_info.clientId not in put_file_recors:
                    put_file_recors[dataset_info.clientId] = PutFileRecord(dataset_info, os.path.abspath(dataset_info.fileName))
                dataset_info.fileName = os.path.basename(dataset_info.fileName)
        for chunk in chunks(inputs, 100):
            get_tickets_response = FileManagementService.getDatasetWriteTickets(chunk)
            all_partial_errors.extend(get_tickets_response.serviceData.partialErrors)
            cls._process_upload_commits(get_tickets_response.commitInfo, put_file_recors)
        return all_partial_errors

    @classmethod
    def _process_upload_commits(cls, commit_infos: List[CommitDatasetFileInfo], put_file_records: Dict[str, PutFileRecord]):
        for comm_info in commit_infos:
            tickets = list()
            file_paths = list()
            for ticket_info in comm_info.datasetFileTicketInfos:
                tickets.append(ticket_info.ticket)
                put_file_info = put_file_records[ticket_info.datasetFileInfo.clientId]
                file_paths.append(put_file_info.physical_path)
            ticket_uids = cls.fcc_client_proxy.register_tickets(tickets)
            try:
                cls.fcc_client_proxy.upload_files_to_plm(ticket_uids, file_paths)
            finally:
                cls.fcc_client_proxy.unregister_tickets(ticket_uids)
        commit_sd = FileManagementService.commitDatasetFiles(commitInput=commit_infos)
        cls._rollback_failed_uploads(commit_sd, put_file_records)

    @classmethod
    def _rollback_failed_uploads(cls, commit_service_data: ServiceData, put_file_records: Dict[str, PutFileRecord]):
        if not commit_service_data.partialErrors:
            return

        failed_client_ids = set(partial_error.clientId for partial_error in commit_service_data.partialErrors)
        tickets = list()
        volume_ids = list()
        for failed_client_id in failed_client_ids:
            put_file_record = put_file_records[failed_client_id]
            tickets.append(put_file_record.ticket_info.ticket)
            volume_ids.append(put_file_record.volume_id)
        ticket_uids = cls.fcc_client_proxy.register_tickets(tickets)
        try:
            cls.fcc_client_proxy.fcc_rollback_files_uploaded_to_plm(ticket_uids, volume_ids)
        finally:
            cls.fcc_client_proxy.unregister_tickets(ticket_uids)



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2020_05.UiConfig import NamedColumnConfigInput, StringMap, ColumnDefInfo, GetNamedColumnConfigsResponse
from tcsoa.gen.Internal.AWS2._2020_05.FileMgmt import GetFilesAndTicketsInfoResponse
from tcsoa.gen.Internal.AWS2._2020_05.DataManagement import RelatedObjectsInfo, GetRelatedObjectsResponse
from tcsoa.gen.Internal.AWS2._2020_05.FullTextSearch import GetSearchSettingsInput, GetSearchSettingsResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getRelatedObjsForConfiguredRevision(cls, input: List[RelatedObjectsInfo]) -> GetRelatedObjectsResponse:
        """
        This operation retrieves the underlying Item or ItemRevision related to source business object.
        
        Use Case:
        
        If Item is attached to Folder in database, Active Workspace client displays the Item as configured
        ItemRevision. The displayed ItemRevision will be as per the revision rule set by the user. If specific
        ItemRevision is attached to Folder in database, that specific ItemRevision is always displayed, irrespective of
        revision rule set by user.
        
        In such cases, underlying object in database is different from the object displayed on client.  When user
        performs a operation on displayed ItemRevision, the actual operation happens on the object related in
        Teamcenter database. 
        For example, if a configured ItemRevision is cut, and its related Item is in database, the Item is likewise cut
        (from database). Similarly, when a specific ItemRevision is cut and the ItemRevision is in the database, only
        the ItemRevision is cut (from database). For business objects other than ItemRevision, if the object displayed
        is identical to object in database, the object in database is cut.
        
        This operation returns actual objects in database for given list of displayed objects on client.
        """
        return cls.execute_soa_method(
            method_name='getRelatedObjsForConfiguredRevision',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetRelatedObjectsResponse,
        )


class FullTextSearchService(TcService):

    @classmethod
    def getSearchSettings(cls, searchSettingInput: GetSearchSettingsInput) -> GetSearchSettingsResponse:
        """
        This operation is used to get the search setting values from the server. These search settings are displayed at
        the client side to make search more configurable and convenient for user. Client needs to render these setting
        in user undersandable formate so that users can customize search based on their requirement.
        
        Use cases:
        Search setting has one of the attributes which is used to configure the category names for which the filters
        are expanded by default after every search.
        Right now these values are configured using preference AWC_Limited_Filter_Categories_Expanded this preference
        takes <typename>.<property name> as input. For example, user needs to set value as WorkspaceObject.object_type
        if only object_type filter category needs to be expanded.
        
        When we are giving an option to configure this value from client, we want user to use display names instead of
        internal names of the type and property to make it more user understandable. Display values in this case will
        be fetched by this new service.
        """
        return cls.execute_soa_method(
            method_name='getSearchSettings',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='FullTextSearch',
            params={'searchSettingInput': searchSettingInput},
            response_cls=GetSearchSettingsResponse,
        )


class UiConfigService(TcService):

    @classmethod
    def loadNamedColumnConfig(cls, uid: str) -> GetNamedColumnConfigsResponse:
        """
        This service operation will load the given named column configuration for a given user.
        
        Use cases:
        Load a Named Column Configuration:
        This service operation will be called which will get the detailed columns information from the selected named
        column configuration.
        """
        return cls.execute_soa_method(
            method_name='loadNamedColumnConfig',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='UiConfig',
            params={'uid': uid},
            response_cls=GetNamedColumnConfigsResponse,
        )

    @classmethod
    def saveNamedColumnConfig(cls, uid: str, columns: List[ColumnDefInfo], namedColumnConfigCriteria: StringMap) -> ServiceData:
        """
        This service operation saves the named column configuration information to the Teamcenter database. Active
        Workspace client may use this information to populate columns of tables in various sublocations as the user
        navigates.
        
        Use cases:
        Save Named Column Configuration:
        A user wants to modify an existing named column configuration for a given table by adding or removing columns.
        The modified named column configuration will become the active column configuration for that user.
        """
        return cls.execute_soa_method(
            method_name='saveNamedColumnConfig',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='UiConfig',
            params={'uid': uid, 'columns': columns, 'namedColumnConfigCriteria': namedColumnConfigCriteria},
            response_cls=ServiceData,
        )

    @classmethod
    def createNamedColumnConfig(cls, namedColumnConfigInput: NamedColumnConfigInput, columns: List[ColumnDefInfo], namedColumnConfigCriteria: StringMap) -> ServiceData:
        """
        This service operation creates the named column configuration information in the Teamcenter database. Active
        Workspace client may use this information to populate columns of tables in various sublocations as the user
        navigates. Default named column configurations can be initially created by a utility during install.
        
        Use cases:
        Create Named Column Configuration:
        A user wants to create a named column configuration for a given table when he/she is logged into the client.
        The named column configuration will become the active column configuration for that user on the given table.
        """
        return cls.execute_soa_method(
            method_name='createNamedColumnConfig',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='UiConfig',
            params={'namedColumnConfigInput': namedColumnConfigInput, 'columns': columns, 'namedColumnConfigCriteria': namedColumnConfigCriteria},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteNamedColumnConfig(cls, uid: str) -> ServiceData:
        """
        This service operation deletes the given named column configuration object from the Teamcenter database.
        
        Use cases:
        Delete Named Column Configuration:
        A user wants to delete an existing named column configuration for a given table. The named column configuration
        will be deleted.
        """
        return cls.execute_soa_method(
            method_name='deleteNamedColumnConfig',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='UiConfig',
            params={'uid': uid},
            response_cls=ServiceData,
        )

    @classmethod
    def getNamedColumnConfigs(cls, namedColumnConfigInput: NamedColumnConfigInput, namedColumnConfigCriteria: StringMap) -> GetNamedColumnConfigsResponse:
        """
        This service operation gets named column configuration information from the Teamcenter database.
        
        Use cases:
        Get a list of Named Column Configurations:
        This service operation will be called to get all available named column configurations for the logged in user.
        """
        return cls.execute_soa_method(
            method_name='getNamedColumnConfigs',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='UiConfig',
            params={'namedColumnConfigInput': namedColumnConfigInput, 'namedColumnConfigCriteria': namedColumnConfigCriteria},
            response_cls=GetNamedColumnConfigsResponse,
        )


class FileMgmtService(TcService):

    @classmethod
    def getFileNamesWithTicketInfo(cls, businessObjectsIn: List[BusinessObject]) -> GetFilesAndTicketsInfoResponse:
        """
        This operation fetches the name and the Teamcenter File Management System (FMS) ticket of the file, related to
        the input business objects of type Dataset, as the primary named reference. In case multiple files are related
        to the Dataset as primary named reference , this operation fetches the first file and its corresponding FMS
        ticket.
        
        This operation can be invoked by the Active Workspace client with at least one of the input objects of type
        Dataset, this means Active Workspace client enables the Download command when at least one of the selected
        object is of type Dataset. This operation handles types other than Dataset gracefully and warns the user for
        the invalid selection.
        
        The primary named reference of the Dataset is defined through its definition using the BMIDE tool of
        Teamcenter. This operation finds the primary named reference using the Tool Actions and the Reference
        definitions of the type Dataset. Tool Actions of the type Dataset defines the mapping of the operation on the
        Dataset with the named reference and the Reference definition of the Dataset maps the named reference and the
        extension of the file. Citing an example , if there are multiple files associated with the named reference i.e
        &lsquo;Text&rsquo; , the first file associated with this named reference is selected. In case , the first file
        is deleted , the second file associated with this named reference takes precedence.
        
        Use cases:
        Use Case 1:
        &#61656;    In Active Workspace client, when user selects  Dataset objects and clicks on the "Download" command
        then the client starts downloading the file directly from the FMS with the help of the FMS ticket information.
        
        Use Case 2:
        &#61656;    In Active Workspace client, user selects the Dataset objects and clicks on the "Copy File Download
        Link" command to copy the URL of the file on the Teamcenter and OS clipboards. When the copied file download
        link is hit by the user on a different page then the client starts downloading the file directly from the FMS
        with the help of the FMS ticket information.
        """
        return cls.execute_soa_method(
            method_name='getFileNamesWithTicketInfo',
            library='Internal-AWS2',
            service_date='2020_05',
            service_name='FileMgmt',
            params={'businessObjectsIn': businessObjectsIn},
            response_cls=GetFilesAndTicketsInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Requirementsmanagement._2009_10.RequirementsManagement import RelativeStructureParentInfo, ImportUnmanagedDataResponse
from typing import List
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def importUnmanagedData(cls, relativeStructParent: List[RelativeStructureParentInfo], importOptions: List[str]) -> ImportUnmanagedDataResponse:
        """
        The SOA operation is used to create and update structures and GRM from MSExcel application. It provides ability
        to create or delete items, GRM and is capable of property editing.
        
        Use cases:
        Create and update item and GRM from exported Excel or from standalone excel sheet.
        
        Exceptions:
        >If the object is modified, New or Deleted from Teamcenter there is any error during generating transient file
        read ticket due to a configuration issue at the server, then the operation throws a service exception. Example-
        If the transient volume directory is not configured at the server then the FMS fails to generate a file at the
        server and subsequent file download operation fails. In such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='importUnmanagedData',
            library='Internal-Requirementsmanagement',
            service_date='2009_10',
            service_name='RequirementsManagement',
            params={'relativeStructParent': relativeStructParent, 'importOptions': importOptions},
            response_cls=ImportUnmanagedDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2014_10.DataManagement import DeepCopyData
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReviseIn(TcBaseObj):
    """
    This structure holds information about the target business object, property names and values and list of deep copy
    data that will be used for the Revise operation input.
    
    :var targetObject: The target business object being revised
    :var inputPropValues: A map (string/ list of strings) that holds input property values. Each value is a list of
    strings to support both single valued and multi valued properties of types. The calling client is responsible for
    converting the different property types (like integer, double, date, etc.) to a string using the appropriate to<
    type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
    :var deepCopyDatas: A list of DeepCopyData objects attached to the target object.
    """
    targetObject: BusinessObject = None
    inputPropValues: PropertyValuesMap3 = None
    deepCopyDatas: List[DeepCopyData] = ()


@dataclass
class SaveAsIn(TcBaseObj):
    """
    This structure holds information about the target business object, property names and values and list of deep copy
    data that will be used for the SaveAs operation.
    
    :var targetObject: The target business object.
    :var inputPropValues: A map (string / list of strings) that holds input property values. Each value is a list of
    strings to support both single valued and multi valued properties of types. The calling client is responsible for
    converting the different property types (like integer, double, date, etc.) to a string using the appropriate to<
    type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
    :var deepCopyDatas: DeepCopyData of the objects attached to the target object.
    """
    targetObject: BusinessObject = None
    inputPropValues: PropertyValuesMap3 = None
    deepCopyDatas: List[DeepCopyData] = ()


"""
This map is of property name (as key) and property values (as value) in string format. Each value is a list of strings to support both single valued and multi valued properties of types. It is used in SaveAsIn structure to hold the property name and values&rsquo; details of the target object. The calling client is responsible for converting the different property types (like integer, double, date, etc) to a string using the appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
"""
PropertyValuesMap3 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0BuildingBlockBOMLine
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MoveLineInfo(TcBaseObj):
    """
    'MoveLineInfo' structure represents the parameters required to re-sequence the selected Fnd0BuildingBlockBOMLine
    object in the BOM structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var operation: The operation integer value to decide the operation performed. Following operation can be supported
    by 'MoveLineInfo'.
    - Move Up
    - Move Down
    - Promote
    - Demote
    - Edit Number 
    
    
    :var selectedLine: The Fnd0BuildingBlockBOMLine object, on which operation will be performed.
    :var newNumber: The new Number property value, will be used when Edit Number operation will be performed.  Based on
    new property value selected BOM line will be moved in the BOM structure.
    """
    clientId: str = ''
    operation: int = 0
    selectedLine: Fnd0BuildingBlockBOMLine = None
    newNumber: str = ''


@dataclass
class MoveLineResponse(TcBaseObj):
    """
    MoveLineResponse structure contains the ServiceData object.
    
    :var serviceData: The Service Data.
    """
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Core._2017_11.LogicalObject import LogicalObjectTypeResponse, AddMemberAndPresentedPropsResponse
from tcsoa.gen.Internal.Core._2018_06.LogicalObject import LogicalObjectTypeInput2, UpdateMemberInput, AddMembersPresentedPropsInput2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class LogicalObjectService(TcService):

    @classmethod
    def updateMembers(cls, updateMemberInputStruct: UpdateMemberInput) -> ServiceData:
        """
        This operation updates a list of logical object member definitions based on the specified input.
        
        Use cases:
        This operation is invoked to update member definitions to an already defined logical object type based on input
        values.
        
        Use Case: 
        
        For example, consider the below use case data for the use case(s) specified: 
        
        Use Case Data:
        Logical object definition 1
        Name                               : "Logical Object 1"
        Root business object   : Item
        
        Member : MemberRevision
        {
            Member ID      : MemberRevision
            Display Name : Revision Member
            Retrieve Classification Data : False
            Navigation Path : vector<TraversalHop2>:REF(fnd0Root, Item).REFBY(items_tag, ItemRevision)
        }
        
        
        Use Case 1: 
        Update member name
        
        UpdateMemberInput
        {
            logicalObjectType = "Logical Object 1"
            membersToBeUpdated = {  < "MemberRevision"
                                                                  MemberPropertyDefinition2
                                                                  { 
                                                                         Member ID : MemberRevision_update
                                                                         Display Name: 
                                                                         Retrieve Classification Data : false
                                                                         Navigation Path:
                                                                  }
                                                             >   
                                                         }                            
        }                                
        
        SOA will update the name of member "MemberRevision" to "MemberRevision_update".
        
        
        Use Case 2: 
        Update member display name and Retrieve classification Data flag
        
        UpdateMemberInput
        {
            logicalObjectType = "Logical Object 1"
            membersToBeUpdated = {  < "MemberRevision"
                                                                  MemberPropertyDefinition2
                                                                  { 
                                                                         Member ID : MemberRevision
                                                                         Display Name: Revision Member update
                                                                         Retrieve Classification Data : true
                                                                         Navigation Path:
                                                                  }
                                                             >   
                                                         }                            
        }                                
        
        SOA will update the display name of member from "Revision Member" to "Revision member update" and update
        retrieve classification data flag to true.
        
        Use Case 3: 
        Update member navigation path
        
        UpdateMemberInput
        {
            logicalObjectType = "Logical Object 1"
            membersToBeUpdated = {  < "MemberRevision"
                                                                  MemberPropertyDefinition2
                                                                  { 
                                                                         Member ID : MemberRevision
                                                                         Display Name: Revision Member update
                                                                         Retrieve Classification Data : false
                                                                         Navigation Path: vector<TraversalHop2>
                                                                  }
                                                             >   
                                                         }                            
        }                                
        
        TraversalHop2[0]
        {
           propertyName = "fnd0Root"
           direction           = "forward"
           destinationType  = "Item"
           propertyType      = ""
           destinationObjectCriteria = ""
        }
        
        TraversalHop2[1]
        {
           propertyName = "IMAN_reference"
           direction  ="forward"
           destinationType  = "Part"
           propertyType     = ""
          destinationObjectCriteria = ""
        } 
        
        SOA will update the navigation path of the member "MemberRevision" to reflect new navigation path.
        
        Use Case 4: 
        Update destinationObjectCriteria of the member navigation path
        
        UpdateMemberInput
        {
            logicalObjectType = "Logical Object 1"
            membersToBeUpdated = {  < "MemberRevision"
                                                                  MemberPropertyDefinition2
                                                                  { 
                                                                         Member ID : MemberRevision
                                                                         Display Name: Revision Member update
                                                                         Retrieve Classification Data : false
                                                                         Navigation Path: vector<TraversalHop2>
                                                                  }
                                                             >   
                                                         }                            
        }                                
        
        TraversalHop2[0]
        {
           propertyName = "IMAN_reference"
           direction           = "forward"
           destinationType  = "ItemRevision"
           propertyType      = "relation"
           destinationObjectCriteria = "$ConfigurationContext"
        }
        
        SOA will update the navigation path of the member "MemberRevision" to reflect new navigation path including
        updated destinationObjectCriteria. Other possible values for destinationObjectCriteria include
        $ConfigurationContext = puid1, and $CurrentUserSessionProject.
        """
        return cls.execute_soa_method(
            method_name='updateMembers',
            library='Internal-Core',
            service_date='2018_06',
            service_name='LogicalObject',
            params={'updateMemberInputStruct': updateMemberInputStruct},
            response_cls=ServiceData,
        )

    @classmethod
    def addMembersAndPresentedProps2(cls, addMembersAndPresentedProps: AddMembersPresentedPropsInput2) -> AddMemberAndPresentedPropsResponse:
        """
        This operation adds a list of members with the capability to select a filter criteria on the destination object
        and presented properties to an existing Logical Object Type based on the specified input. 
        A Logical Object Type is a sub-type of Fnd0LogicalObject.
        
        Use cases:
        This operation is invoked to add members and presented properties to an existing Logical Object Type.
        Configuration Context can optionally be added to the Traversal Hop structure for filtering during Logical
        Object retrieval.
        
        Use Case 1: 
        The user wants to add a member to a Logical Object Type (subtype of Fnd0LogicalObject)  with default
        Configuration Context criteria specified.
        Note the input value for destinationObjectCriteria  must be the selected value from Dynamic Filter Criteria LOV.
        
        TraversalHop2 = {
            propertyName            = IMAN_reference; 
        propertyType            = relation;
        destinationType            = ItemRevision; 
        direction            = forward; 
        destinationObjectCriteria    = $ConfigurationContext; 
        }
        
        Use Case 2: 
        The user wants to add a member to a Logical Object Type (subtype of Fnd0LogicalObject)  with a specific
        Configuration Context criteria specified.
        
        TraversalHop2 = {
            propertyName            = IMAN_reference; 
        propertyType            = relation;
        destinationType            = ItemRevision; 
        direction            = forward; 
        destinationObjectCriteria    = $ConfigurationContext = puid1; 
        }
        
        Use Case 3: 
        The user wants to add a member to a Logical Object Type (subtype of Fnd0LogicalObject)  with the Current User
        Session Project criteria specified.
        
        TraversalHop2 = {
            propertyName            = IMAN_reference; 
        propertyType            = relation;
        destinationType            = Item; 
        direction            = forward; 
        destinationObjectCriteria    = $CurrentUserSessionProject; 
        }
        
        Use Case 4: 
        The user wants to add a member to a Logical Object Type (subtype of Fnd0LogicalObject)  with default
        Configuration Context and the Current User Session Project criteria specified.
        
        TraversalHop2 = {
            propertyName            = IMAN_reference; 
        propertyType            = relation;
        destinationType            = Item; 
        direction            = forward; 
        destinationObjectCriteria    = $CurrentUserSessionProject  and   
           $ConfigurationContext; 
        }
        """
        return cls.execute_soa_method(
            method_name='addMembersAndPresentedProps2',
            library='Internal-Core',
            service_date='2018_06',
            service_name='LogicalObject',
            params={'addMembersAndPresentedProps': addMembersAndPresentedProps},
            response_cls=AddMemberAndPresentedPropsResponse,
        )

    @classmethod
    def createLogicalObjectTypes2(cls, logicalObjectTypeInputs: List[LogicalObjectTypeInput2]) -> LogicalObjectTypeResponse:
        """
        This operation creates a list of Logical Object Types (subtypes of Fnd0LogicalObject) based on the specified
        input. The name, display name, description and root type are the mandatory fields required to create a Logical
        Object Type. This operation has the capabiltiy to add a list of members and to select a filter criteria on the
        destination object and presented properties to an existing Logical Object Type.
        A Logical Object Type is created only if its definition data satisfies the pre-defined constraints.
        
        Use cases:
        This operation is invoked to create Logical Object Types (subtypes of Fnd0LogicalObject)  after obtaining the
        user input fields from the logical object creation dialog.
        
        Use Case 1: 
        The user wants to create a Logical Object Type(subtype of Fnd0LogicalObject)  with a root business object and
        without member and presented properties. In this case the input structure to create a Logical Object Type would
        look like the following
        LogicalObjectTypeInput = {
        name = loName; 
        displayName = logicalObjectName;
        description = Logical Object Type; 
        rootType = Item; 
        parentType = Fnd0LogicalObject;
        }
        Use Case 2: 
        The user wants to create a Logical Object Type with root business object, member and presented properties as
        below
        1.root business object : Item 
        2.member object: ItemRevision 
        3.presented properties : item_revision_id and item_id. 
        In this case the input structures to create a Logical Object Type with member and presented property
        definitions would look like the following
        LogicalObjectTypeInput2 = {
        name = loName; 
        displayName= logicalObjectName;
        description = Logical Object Type; 
        rootType = Item; 
        parentType = Fnd0LogicalObject; 
        memberPropertyDefinitions = exampleMemberPropertyDefinitions;
        presentedPropertyDefinitions = examplePresentedPropertyDefinitions;
        }
        exampleMemberPropertyDefinitions[0] = {
        memberPropertyName = itemRevisionMember;
        displayName= Item Revision;
        description= Item Revision members;
        traversalPath = exampleTraversalPaths;
        }
        exampleTraversalPaths [0] = {
        propertyName = fnd0Root
        propertyType = refrence
        destinationType = Item
        direction = forward
        destinationObjectCriteria = USER
        }
        exampleTraversalPaths [1] = {
        propertyName = items_tag
        propertyType = reference
        destinationType = ItemRevision
        direction = reverse
        destinationObjectCriteria = 
        }
        examplePresentedPropertyDefinitions[0] = {
        presentedPropertyName = ItemRevisionId; 
        displayName = ItemRevisionId; 
        description = Presented property from member: ItemRevissionId; 
        rootOrMemberName = itemRevisionMember;
        sourcePropertyName = item_revision_id;
        }
        examplePresentedPropertyDefinitions[1] = {
        presentedPropertyName = ItemId; 
        displayName = ItemId; 
        description = Presented property from root: ItemId; 
        rootOrMemberName = fnd0Root;
        sourcePropertyName = item_id;
        """
        return cls.execute_soa_method(
            method_name='createLogicalObjectTypes2',
            library='Internal-Core',
            service_date='2018_06',
            service_name='LogicalObject',
            params={'logicalObjectTypeInputs': logicalObjectTypeInputs},
            response_cls=LogicalObjectTypeResponse,
        )



from __future__ import annotations

from tcsoa.gen.ChangeManagement._2009_06.ChangeManagement import GetNoteVariantsInput, GetNoteVariantResponse
from typing import List
from tcsoa.utils import TcService


class ChangeManagementService(TcService):

    @classmethod
    def getNoteVariantChanges(cls, getNoteVariantRequest: List[GetNoteVariantsInput]) -> GetNoteVariantResponse:
        """
        This operation is specifically designed to handle the retrieval of information for Note or Variant changes to
        be consumed by the Teamcenter Structure Manager Rich Client UI.  There are helper functions in the Rich Client
        to facilitate the consumption and interpretation of the retrieved information.  In other words, this operation
        may pose challenges to users of this operation who are unfamiliar with the intended usage of the returned
        details.  For Rich Client developers, it is better to use the helper functions instead.
        The operation accepts as input a list of 'GetNoteVariantsInput' structures, each containing an object reference
        to a 'BOMEdit' whose integer type is EITHER 6 (=Note Change) OR 7(=Variant Change) and a matching context
        string of one of the following two possible values:
        - CM_note_change_details
        - CM_variant_change_details
        
        
        Based on the input structures, the operation will assemble the retrieved information in a list of
        'GetNoteVariantOutput' structures, and package them together with a standard service data in the returned
        'GetNoteVariantResponse' structure.
        
        Use cases:
        Use Case 1: Getting the details for a note change
        This operation can be invoked via an instance of the 'ChangeManagementService'.  The caller program will need
        to supply an object reference to a 'BOMEdit' whose integer type is 6 and a matching context string of
        CM_note_change_details in the input structure 'GetNoteVariantsInput'.  The corresponding output structure
        'GetNoteVariantOutput' contains object references to 1) the BOMEdit, 2) the associated change revision, 3) the
        solution bvr, and 4) the impacted bvr.  It also contains a list of details count and a list of strings
        representing some textual details of the note change.  The caller program will use the count to read the
        strings for details.
        Use Case 2: Getting the details for a variant change
        This operation can be invoked via an instance of the 'ChangeManagementService'.  The caller program will need
        to supply an object reference to a 'BOMEdit' whose integer type is 7 and a matching context string of
        CM_variant_change_details in the input structure 'GetNoteVariantsInput'.  The corresponding output structure
        'GetNoteVariantOutput' contains object references to 1) the BOMEdit, 2) the associated change revision, 3) the
        solution bvr, and 4) the impacted bvr.  It also contains a list of details count and a list of strings
        representing some textual details of the variant change.  The caller program will use the count to read the
        strings for details.
        
        """
        return cls.execute_soa_method(
            method_name='getNoteVariantChanges',
            library='ChangeManagement',
            service_date='2009_06',
            service_name='ChangeManagement',
            params={'getNoteVariantRequest': getNoteVariantRequest},
            response_cls=GetNoteVariantResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2020_04.DataManagement import GenerateContextIDsInput2
from tcsoa.gen.Core._2016_05.DataManagement import GenerateContextSpecificIDsResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def generateContextSpecificIDs2(cls, generateContextIDsIn: List[GenerateContextIDsInput2]) -> GenerateContextSpecificIDsResponse:
        """
        Generates the range of unique IDs for input context names. The number of IDs generated for each context name
        depends on the input. If for a given context name, the ID has been reset using Teamcenter service
        resetContextID, then this service generates IDs for that context from the beginning.
        
        ID generation will also reset when the maximum limit is met. This limit is maximum number supported on 64 bit
        machine.
        
        If the validation flag is true and both contextType and contextPropName are provided, the generated IDs will be
        validated for uniquness.
        """
        return cls.execute_soa_method(
            method_name='generateContextSpecificIDs2',
            library='Core',
            service_date='2020_04',
            service_name='DataManagement',
            params={'generateContextIDsIn': generateContextIDsIn},
            response_cls=GenerateContextSpecificIDsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2018_11.ResourceManagement import AutoPositionComponentByCSYSResponse
from tcsoa.gen.Internal.Manufacturing._2018_11.StructureManagement import FindBrokenPartsInPVInputInfo, EvaluateLinksResponse, FindBrokenProductViewsResponse, FindBrokenProductViewsInputInfo, EvaluateLinksData, FindBrokenPartsInPVResponse
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def evaluateLinks(cls, input: List[EvaluateLinksData]) -> EvaluateLinksResponse:
        """
        This service operation converts the URL representing a BOMLine to a BOMLine instance. Additionally, returns a
        structure which encapsulates the CCObject or StructureContext representing the context. The parent chain of the
        BOMLine are returned in the serviceData.
        
        Use cases:
        A user can capture the links from Teamcenter representing the BOMLine objects using a utility like
        tc_to_intosite, and use those links with this API to reconstruct the BOMLine objects that were used to create
        the links. These BOMLine objects will have the same configuration and context as the time of the link creation.
        """
        return cls.execute_soa_method(
            method_name='evaluateLinks',
            library='Internal-Manufacturing',
            service_date='2018_11',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=EvaluateLinksResponse,
        )

    @classmethod
    def findBrokenPartsInProductView(cls, input: FindBrokenPartsInPVInputInfo) -> FindBrokenPartsInPVResponse:
        """
        This operation finds the one or more broken parts in the input Dataset product view (PV)objects.
        For a broken PV, it is not possible for the viewer to be able to resolve some or all parts saved during its
        creation. The supplied PV contains such unresolved parts which needs to be updated.
        The operation provides interface to identify broken parts in a PV, so the User can take appropriate actions on
        them.
        
        Use cases:
        User creates a broken PV either by restructuring or deleting elements of BOMLine structure.
        User invokes the service operation with a broken PV Dataset. The operation returns the list of all broken parts
        for given input.
        """
        return cls.execute_soa_method(
            method_name='findBrokenPartsInProductView',
            library='Internal-Manufacturing',
            service_date='2018_11',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=FindBrokenPartsInPVResponse,
        )

    @classmethod
    def findBrokenProductViews(cls, input: FindBrokenProductViewsInputInfo) -> FindBrokenProductViewsResponse:
        """
        This operation finds the one or more broken product views (PV) in the input BOMLine objects.
        For a broken PV, it is not possible for the viewer to be able to resolve some or all parts saved during its
        creation. The supplied PV contains such unresolved parts which needs to be updated.
        The operation provides interface to identify broken PVs and their associated BOMLine objects, so the User can
        take appropriate actions on them.
        
        Use cases:
        User creates a broken PV either by restructuring or deleting elements of BOMLine structure.
        User invokes the service operation with BOMLine objects containing such broken PVs.
        The operation returns the list of all such broken PVs for the given scopes as well as their attached BOMLine
        objects.
        """
        return cls.execute_soa_method(
            method_name='findBrokenProductViews',
            library='Internal-Manufacturing',
            service_date='2018_11',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=FindBrokenProductViewsResponse,
        )


class ResourceManagementService(TcService):

    @classmethod
    def autoPositionComponentByCSYS(cls, strSourceBOMLines: List[str], strTargetBOMLine: str, strSourceCSYS: str, strTargetCSYS: str) -> AutoPositionComponentByCSYSResponse:
        """
        This operation positions a newly added target component so that it fits to the source component geometrically
        based on coordinate systems (CSYSs). This opeation is used when the user 
        a) adds a classified component, 
        b) adds a component with Guided Component Search (GCS) or 
        c) adds a component with copy and paste 
        in Teamcenter Rich Application Client (RAC).
        The source object can be a component, a CSYS or a GCS Connection Point (CP).
        
        This operation analyses the source and target CSYSs and GCS CPs and does the geometrical positioning by setting
        the transformation matrix in the target component BOMLine. (In one specific case (GCS Add Upwards), the target
        component is positioned to the zero position and the source position is adjusted correspondingly.)
        
        If the source object is a component or a GCS CP, it can happen that there are multiple possible source CSYSs.
        In this case the operation returns those CSYSs in the response object and does NOT autoposition, if no specific
        source CSYS was specified. The operation has to be called in that case again with an unique source CSYS.
        
        If the target object (always a component) has multiple possible target CSYSs, the operation returns those CSYSs
        in the response object and does NOT autoposition, if no specific target CSYS was specified. The operation has
        to be called in this case again with an unique target CSYS.
        
        Use cases:
        When the user creates resource assembly in Resource Manager components will be positioned correctly if both
        parent and child components have appropriate CSYSs. The user can do add/paste/GCS add components by selecting a
        component, CSYS or selecting one or multiple GCS CP lines. The system then analyzes the CSYSs under the parent
        and child component and carry out the positioning. If there are appropriate multiple CSYS for positioning, the
        system asks the user to select one of them.
        """
        return cls.execute_soa_method(
            method_name='autoPositionComponentByCSYS',
            library='Internal-Manufacturing',
            service_date='2018_11',
            service_name='ResourceManagement',
            params={'strSourceBOMLines': strSourceBOMLines, 'strTargetBOMLine': strTargetBOMLine, 'strSourceCSYS': strSourceCSYS, 'strTargetCSYS': strTargetCSYS},
            response_cls=AutoPositionComponentByCSYSResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Core._2017_05.FileManagement import DatashareManagerReplaceInfo
from tcsoa.gen.Internal.Core._2014_10.FileManagement import GetPlmdFileTicketResponse
from tcsoa.gen.Internal.Core._2017_05.PresentationManagement import GetStylesheetPerPageResponse, StylesheetPerPageInputData, GetStylesheetPerPagePref
from typing import List
from tcsoa.utils import TcService


class PresentationManagementService(TcService):

    @classmethod
    def getStylesheetPerPage(cls, pref: GetStylesheetPerPagePref, inputData: List[StylesheetPerPageInputData]) -> GetStylesheetPerPageResponse:
        """
        This operation gets the stylesheet data and list of names of the available visible pages for the input object.
        The stylesheet data will be returned only for the target page which is defined in the input structure. If
        target page is not specified, then it will return stylesheet data for first visible page. This helps to improve
        client performance by querying only for the information that will be displayed.
        The clients are expected to call the operation without a target page specified. And then, if the user selects a
        page to display, then make another call to the operation with the selected page name as the target page in the
        input. 
        
        If pref.processEntireXRT = true, this operation will process the data for the entire XRT stylesheet.  If false,
        this operation will only process the target page which is defined in the input structure.
        
        This operation also converts XRT stylesheets to HTML stylesheets and returns the stylesheet data to the client.
        If the Teamcenter preference "UsePropertyStylesheetPlatformRenderer" is set to "true", then only XRT is
        returned and no attempt to convert to/return HTML is made. This service accepts a "returnHTMLOnly" preference
        which indicates that the caller can only consume HTML, and overrides the
        "UsePropertyStylesheetsPlatformRenderer" preference, to enable conversion to HTML. This operation only returns
        XRT or HTML, never both values to the client.
        
        If the XRT to be presented in the UI contains objectSet elements, which display an object's children in the UI,
        then those children are returned in the output childrenMap container, and their thumbnail file tickets are
        added to the FileTicketMap, as required.
        
        Common uses for this operation would be to retrieve stylesheet data in either XRT or HTML format for a selected
        object in the UI, in order to present that object to the user.
        
        Use cases:
        Use case 1:
        1.    The user chooses the summary tab in the RichClient.
        2.    The user selects an object in the Navigator pane in RichClient.
        3.    The getStylesheetPerPage operation is invoked and the resulting stylesheet is presented in the UI.
        
        Use case 2:
        1. The user selects an object in the Navigator pane in RichClient.
        2.    The user chooses to view properties for that object.
        3.    The getStylesheetPerPage operation is invoked and the resulting stylesheet is presented in the UI.
        
        Use case 3:
        1. The user chooses the summary tab in the RichClient.
        2.    The user selects an object in the Navigator pane in the RichClient.
        3.    The getStylesheetPerPage operation is invoked and the resulting stylesheet is presented in the UI.
                3.1. The operation also returns the list of names of the visible pages, for example
                       "tc_xrt_Overview", "tc_xrt_AttachedFiles", "tc_xrt_RelatedLinks", "tc_xrt_AuditLogs" etc.
        4.    The use clicks on the "Audit Logs" page in the summary tab. The getStylesheetPerPage operation           
             
                is invoked, with the target page set to "Audit Logs".
        5.    The getStylesheetPerPage operation returns the stylesheet data for the audit log page and it is presented
        in the UI.
        """
        return cls.execute_soa_method(
            method_name='getStylesheetPerPage',
            library='Internal-Core',
            service_date='2017_05',
            service_name='PresentationManagement',
            params={'pref': pref, 'inputData': inputData},
            response_cls=GetStylesheetPerPageResponse,
        )


class FileManagementService(TcService):

    @classmethod
    def getPlmdFileTicketForReplace(cls, infos: List[DatashareManagerReplaceInfo]) -> GetPlmdFileTicketResponse:
        """
        This operation generates a File Management System (FMS) transient read ticket for Product Lifecycle Management
        Data (PLMD) file.
        The PLMD file is used by Datashare Manager Application to asynchronously replace one or more PLM files with
        files from a local client environment, which are uploaded into the Teamcenter volume. The PLMD file details
        like FMS Bootstrap URLs, dataset names, type, reference names, original file names, and absolute file paths and
        write tickets.
        
        Use cases:
        Active Workspace client initiates an asynchrouous file replace.
        """
        return cls.execute_soa_method(
            method_name='getPlmdFileTicketForReplace',
            library='Internal-Core',
            service_date='2017_05',
            service_name='FileManagement',
            params={'infos': infos},
            response_cls=GetPlmdFileTicketResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Ai._2016_03.Ai import GetAITypesResponse
from typing import List
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def getApplicationInterfaceTypes(cls, specificTypeNames: List[str]) -> GetAITypesResponse:
        """
        This operation returns all  the AppInterfaceType objects in the system. If specific names are provided in the
        input, then only the matching AppInterfaceType objects are returned.
        """
        return cls.execute_soa_method(
            method_name='getApplicationInterfaceTypes',
            library='Internal-Ai',
            service_date='2016_03',
            service_name='Ai',
            params={'specificTypeNames': specificTypeNames},
            response_cls=GetAITypesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2016_12.RequirementsManagement import ImportExportOptions
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class BaselineInputData(TcBaseObj):
    """
    The structure BaselineInputData represents the input data required to baseline selected object.
    
    :var objectToBaseline: A business object (ItemRevision or Awb0Element) to be baselined.
    :var baselineRevisionId: The required revision ID of new Baseline Revision which would be created after baseline
    completes. This is an optional parameter and can be empty.
    :var baselineDescription: Description for the new Baseline Revision to be created. This is an optional parameter
    and can be empty.
    :var baselineReleaseProcedureName: Name of the Release Procedure through which the Baseline Revision should
    undergo. The possible release procedure name is governed by preference Baseline_release_procedures.
    :var baselineJobName: Name of the Job for new Baseline Revision to be created. This is an optional parameter and
    can be empty.
    :var baselineLabelName:   Name of the Snapshot folder . Snapshot folder name  is required for imprecise
    baselining.This field is optional and can be blank.
    :var baselineJobDescription: Description of the Job. This is an optional parameter and can be empty.
    :var isDryRun: This option indicates whether dry run creation is to be performed. This is an  optional parameter
    and can be empty. The current value is defaulted to false. This parameter is for future use.
    :var isPreciseBaseline: This option indicates whether the created structure would be a precise or imprecise.  Value
    true denotes precise structure and false denotes imprecise structure. The default value is true.
    """
    objectToBaseline: BusinessObject = None
    baselineRevisionId: str = ''
    baselineDescription: str = ''
    baselineReleaseProcedureName: str = ''
    baselineJobName: str = ''
    baselineLabelName: str = ''
    baselineJobDescription: str = ''
    isDryRun: bool = False
    isPreciseBaseline: bool = False


@dataclass
class BaselineInputDataAsync(TcBaseObj):
    """
    This is required as an input for operation createBaselineAsync
    
    :var baselineInputDataSync: Sync Baseline Input Data.
    :var recipeObject: The recipe of selected BOMLine object which contains information to reconstruct it back and
    create baseline.
    """
    baselineInputDataSync: BaselineInputData = None
    recipeObject: str = ''


@dataclass
class ObjectTemplateInputs(TcBaseObj):
    """
    This structure contains additional options required to be passed when object template is applied during export.
    
    :var boType: Type of object. The supported types are ItemRevision and its sub-types.
    :var objectTemplateName: Name of the object template to be applied during export.
    """
    boType: str = ''
    objectTemplateName: str = ''


@dataclass
class ExportToApplicationInputData4(TcBaseObj):
    """
    The ExportToApplicationInput4 represents the data required to export selected objects to MSWord or MSExcel.
    
    :var objectsToExport: A list of Teamcenter business objects to be exported.
    :var targetObjectsToExport: The list of Teamcenter business objects to be exported which acts as target objects
    used for comparing the contents. An optional parameter and can be empty.
    :var recipeSourceObjects: The recipe of selected BOMLine objects which contains information to reconstruct them
    back and export. This is an optional parameter and can be empty.
    :var recipeTargetObjects: The recipe of target BOMLine objects which contains information to reconstruct them back
    and export. This is an optional parameter used for compare scenarios and can be empty.
    :var attributesToExport: A list of attributes that are visible in Active Workspace for export. An optional
    parameter which can be empty. (For future use.)
    :var applicationFormat: The application formats supported are:  "MSExcel", "MSWordXMLLive", "MExcelLive",
    "MSExcelReimport", "MSWordCompare", "HTML".
    :var templateName: The name of the MSWord or MSExcel template.
    :var exportOptions: List of options for export. Supported options are: 
    "CheckOutObjects" - the selected object and its hierarchy is checked out during export.
    "RunInbackground" - the export operation will be executed in background and a notification is sent when export is
    completed.
    :var objectTemplateInputs: Inputs for applying object template during export.
    :var includeAttachments: If true, the attachments are to be included while exporting.
    """
    objectsToExport: List[BusinessObject] = ()
    targetObjectsToExport: List[BusinessObject] = ()
    recipeSourceObjects: List[str] = ()
    recipeTargetObjects: List[str] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateName: str = ''
    exportOptions: List[ImportExportOptions] = ()
    objectTemplateInputs: List[ObjectTemplateInputs] = ()
    includeAttachments: bool = False



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDisplayStringsOutput(TcBaseObj):
    """
    The name/value pair of a Text Server key and its corresponding localized value.
    
    :var key: The textserver key.
    :var value: The localized value for the Text Server key.
    """
    key: str = ''
    value: str = ''


@dataclass
class GetDisplayStringsResponse(TcBaseObj):
    """
    The response for  the 'getDisplayStrings' operation.
    
    :var output: A list Text Server key/localized value pairs.
    :var serviceData: Partial errors with the key name attached to the partial error.
    """
    output: List[GetDisplayStringsOutput] = ()
    serviceData: ServiceData = None
